from ._utils import api_util
from .Bases import Iterable
from dss import ActionCodes


class IReclosers(Iterable):
    __slots__ = []

    __name__ = "Reclosers"
    _api_prefix = "Reclosers"
    _columns = [
        "Name",
        "Idx",
        "GroundInst",
        "GroundTrip",
        "MonitoredObj",
        "MonitoredTerm",
        "SwitchedObj",
        "SwitchedTerm",
        "NumFast",
        "PhaseInst",
        "PhaseTrip",
        "RecloseIntervals",
        "Shots",
        "State",
        "NormalState",
    ]

    def Close(self):
        self._lib.Reclosers_Close()

    def Open(self):
        self._lib.Reclosers_Open()

    def GroundInst(self, *args):
        """
        Ground (3I0) instantaneous trip setting - curve multiplier or actual amps.

        Original COM help: https://opendss.epri.com/GroundInst.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Reclosers_Get_GroundInst()

        # Setter
        (Value,) = args
        self._lib.Reclosers_Set_GroundInst(Value)

    def GroundTrip(self, *args):
        """
        Ground (3I0) trip multiplier or actual amps

        Original COM help: https://opendss.epri.com/GroundTrip.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Reclosers_Get_GroundTrip()

        # Setter
        (Value,) = args
        self._lib.Reclosers_Set_GroundTrip(Value)

    def MonitoredObj(self, *args):
        """
        Full name of object this Recloser to be monitored.

        Original COM help: https://opendss.epri.com/MonitoredObj2.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Reclosers_Get_MonitoredObj()

        # Setter
        (Value,) = args
        self._lib.Reclosers_Set_MonitoredObj(Value)

    def MonitoredTerm(self, *args):
        """
        Terminal number of Monitored object for the Recloser

        Original COM help: https://opendss.epri.com/MonitoredTerm2.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Reclosers_Get_MonitoredTerm()

        # Setter
        (Value,) = args
        self._lib.Reclosers_Set_MonitoredTerm(Value)

    def NumFast(self, *args):
        """
        Number of fast shots

        Original COM help: https://opendss.epri.com/NumFast.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Reclosers_Get_NumFast()

        # Setter
        (Value,) = args
        self._lib.Reclosers_Set_NumFast(Value)

    def PhaseInst(self, *args):
        """
        Phase instantaneous curve multiplier or actual amps

        Original COM help: https://opendss.epri.com/PhaseInst.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Reclosers_Get_PhaseInst()

        # Setter
        (Value,) = args
        self._lib.Reclosers_Set_PhaseInst(Value)

    def PhaseTrip(self, *args):
        """
        Phase trip curve multiplier or actual amps

        Original COM help: https://opendss.epri.com/PhaseTrip.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Reclosers_Get_PhaseTrip()

        # Setter
        (Value,) = args
        self._lib.Reclosers_Set_PhaseTrip(Value)

    def RecloseIntervals(self):
        """
        Array of Doubles: reclose intervals, s, between shots.

        Original COM help: https://opendss.epri.com/RecloseIntervals.html
        """
        return self._lib.Reclosers_Get_RecloseIntervals_GR()

    def Shots(self, *args):
        """
        Number of shots to lockout (fast + delayed)

        Original COM help: https://opendss.epri.com/Shots.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Reclosers_Get_Shots()

        # Setter
        (Value,) = args
        self._lib.Reclosers_Set_Shots(Value)

    def SwitchedObj(self, *args):
        """
        Full name of the circuit element that is being switched by the Recloser.

        Original COM help: https://opendss.epri.com/SwitchedObj1.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Reclosers_Get_SwitchedObj()

        # Setter
        (Value,) = args
        self._lib.Reclosers_Set_SwitchedObj(Value)

    def SwitchedTerm(self, *args):
        """
        Terminal number of the controlled device being switched by the Recloser

        Original COM help: https://opendss.epri.com/SwitchedTerm1.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Reclosers_Get_SwitchedTerm()

        # Setter
        (Value,) = args
        self._lib.Reclosers_Set_SwitchedTerm(Value)

    def Reset(self):
        """
        Reset recloser to normal state.
        If open, lock out the recloser.
        If closed, resets recloser to first operation.
        """
        self._lib.Reclosers_Reset()

    def State(self, *args):
        """
        Present state of recloser.
        If set to open (ActionCodes.Open=1), open recloser's controlled element and lock out the recloser.
        If set to close (ActionCodes.Close=2), close recloser's controlled element and resets recloser to first operation.
        """
        # Getter
        if len(args) == 0:
            return self._lib.Reclosers_Get_State()

        # Setter
        (Value,) = args
        self._lib.Reclosers_Set_State(Value)

    def NormalState(self, *args):
        """
        Normal state (ActionCodes.Open=1, ActionCodes.Close=2) of the recloser.

        Original COM help: https://opendss.epri.com/NormalState1.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Reclosers_Get_NormalState()

        # Setter
        (Value,) = args
        self._lib.Reclosers_Set_NormalState(Value)


_Reclosers = IReclosers(api_util)

# For backwards compatibility, bind to the default instance
AllNames = _Reclosers.AllNames
Close = _Reclosers.Close
Count = _Reclosers.Count
First = _Reclosers.First
GroundInst = _Reclosers.GroundInst
GroundTrip = _Reclosers.GroundTrip
Idx = _Reclosers.Idx
MonitoredObj = _Reclosers.MonitoredObj
MonitoredTerm = _Reclosers.MonitoredTerm
Name = _Reclosers.Name
Next = _Reclosers.Next
NormalState = _Reclosers.NormalState
NumFast = _Reclosers.NumFast
Open = _Reclosers.Open
PhaseInst = _Reclosers.PhaseInst
PhaseTrip = _Reclosers.PhaseTrip
RecloseIntervals = _Reclosers.RecloseIntervals
Reset = _Reclosers.Reset
Shots = _Reclosers.Shots
State = _Reclosers.State
SwitchedObj = _Reclosers.SwitchedObj
SwitchedTerm = _Reclosers.SwitchedTerm
_columns = _Reclosers._columns
__all__ = [
    "AllNames",
    "Close",
    "Count",
    "First",
    "GroundInst",
    "GroundTrip",
    "Idx",
    "MonitoredObj",
    "MonitoredTerm",
    "Name",
    "Next",
    "NormalState",
    "NumFast",
    "Open",
    "PhaseInst",
    "PhaseTrip",
    "RecloseIntervals",
    "Reset",
    "Shots",
    "State",
    "SwitchedObj",
    "SwitchedTerm",
]



from ._utils import api_util
from .Bases import Iterable


class IWindGens(Iterable):
    """WindGen objects"""

    __slots__ = []
    __name__ = "WindGens"
    _api_prefix = "WindGens"
    _columns = [
        "Name",
        "Idx",
        "RegisterNames",
        "RegisterValues",
        "Ag",
        "Bus1",
        "Class",
        "Cp",
        "daily",
        "duty",
        "IsDelta",
        "kV",
        "kVA",
        "kvar",
        "kW",
        "Lamda",
        "N_WTG",
        "NPoles",
        "pd",
        "PF",
        "Phases",
        "PSS",
        "QFlag",
        "QMode",
        "QSS",
        "Rad",
        "RThev",
        "VCutIn",
        "VCutOut",
        "Vss",
        "WindSpeed",
        "XThev",
        "Yearly",
    ]

    def RegisterNames(self):
        """
        Array of WindGen energy meter register names

        See also the enum `GeneratorRegisters`.
        """
        return self._lib.WindGens_Get_RegisterNames()

    def RegisterValues(self):
        """Array of values in WindGen registers."""
        return self._lib.WindGens_Get_RegisterValues_GR()

    def kV(self, *args):
        """
        Nominal rated (1.0 per unit) voltage for the active WindGen, in kV.
        """
        # Getter
        if len(args) == 0:
            return self._lib.WindGens_Get_kV()

        # Setter
        (Value,) = args
        self._lib.WindGens_Set_kV(Value)

    def kvar(self, *args):
        """
        Base kvar for the active WindGen.
        """
        # Getter
        if len(args) == 0:
            return self._lib.WindGens_Get_kvar()

        # Setter
        (Value,) = args
        self._lib.WindGens_Set_kvar(Value)

    def kW(self, *args):
        """
        Total base kW for the active WindGen.
        """
        # Getter
        if len(args) == 0:
            return self._lib.WindGens_Get_kW()

        # Setter
        (Value,) = args
        self._lib.WindGens_Set_kW(Value)

    def PF(self, *args):
        """
        WindGen power factor. Power factor (pos. = producing vars).
        """
        # Getter
        if len(args) == 0:
            return self._lib.WindGens_Get_PF()

        # Setter
        (Value,) = args
        self._lib.WindGens_Set_PF(Value)

    def kVA(self, *args):
        """
        KVA rating of the electrical machine in the WindGen.
        """
        # Getter
        if len(args) == 0:
            return self._lib.WindGens_Get_kVA()

        # Setter
        (Value,) = args
        self._lib.WindGens_Set_kVA(Value)

    def Ag(self, *args):
        """
        Gearbox ratio
        """
        # Getter
        if len(args) == 0:
            return self._lib.WindGens_Get_Ag()

        # Setter
        (Value,) = args
        self._lib.WindGens_Set_Ag(Value)

    def Cp(self, *args):
        """
        Turbine performance coefficient.
        """
        # Getter
        if len(args) == 0:
            return self._lib.WindGens_Get_Cp()

        # Setter
        (Value,) = args
        self._lib.WindGens_Set_Cp(Value)

    def Lamda(self, *args):
        """
        Tip speed ratio
        """
        # Getter
        if len(args) == 0:
            return self._lib.WindGens_Get_Lamda()

        # Setter
        (Value,) = args
        self._lib.WindGens_Set_Lamda(Value)

    def N_WTG(self, *args):
        """
        Number of WTG in aggregation
        """
        # Getter
        if len(args) == 0:
            return self._lib.WindGens_Get_N_WTG()

        # Setter
        (Value,) = args
        self._lib.WindGens_Set_N_WTG(Value)

    def NPoles(self, *args):
        """
        Number of pole pairs of the induction generator
        """
        # Getter
        if len(args) == 0:
            return self._lib.WindGens_Get_NPoles()

        # Setter
        (Value,) = args
        self._lib.WindGens_Set_NPoles(Value)

    def pd(self, *args):
        """
        Air density in kg/m3
        """
        # Getter
        if len(args) == 0:
            return self._lib.WindGens_Get_pd()

        # Setter
        (Value,) = args
        self._lib.WindGens_Set_pd(Value)

    def PSS(self, *args):
        """
        Steady state output real power.
        """
        # Getter
        if len(args) == 0:
            return self._lib.WindGens_Get_PSS()

        # Setter
        (Value,) = args
        self._lib.WindGens_Set_PSS(Value)

    def QFlag(self, *args):
        """
        Non-zero values enable reactive power and voltage control in the dynamic model.
        """
        # Getter
        if len(args) == 0:
            return self._lib.WindGens_Get_QFlag()

        # Setter
        (Value,) = args
        self._lib.WindGens_Set_QFlag(Value)

    def QMode(self, *args):
        """
        Q control mode (0:Q, 1:PF, 2:VV).
        """
        # Getter
        if len(args) == 0:
            return self._lib.WindGens_Get_QMode()

        # Setter
        (Value,) = args
        self._lib.WindGens_Set_QMode(Value)

    def QSS(self, *args):
        """
        Steady state output reactive power.
        """
        # Getter
        if len(args) == 0:
            return self._lib.WindGens_Get_QSS()

        # Setter
        (Value,) = args
        self._lib.WindGens_Set_QSS(Value)

    def Rad(self, *args):
        """
        Rotor radius in meters
        """
        # Getter
        if len(args) == 0:
            return self._lib.WindGens_Get_Rad()

        # Setter
        (Value,) = args
        self._lib.WindGens_Set_Rad(Value)

    def RThev(self, *args):
        """
        Per unit Thevenin equivalent resistance (R).
        """
        # Getter
        if len(args) == 0:
            return self._lib.WindGens_Get_RThev()

        # Setter
        (Value,) = args
        self._lib.WindGens_Set_RThev(Value)

    def VCutIn(self, *args):
        """
        Cut-in speed for the wind generator
        """
        # Getter
        if len(args) == 0:
            return self._lib.WindGens_Get_VCutIn()

        # Setter
        (Value,) = args
        self._lib.WindGens_Set_VCutIn(Value)

    def VCutOut(self, *args):
        """
        Cut-out speed for the wind generator
        """
        # Getter
        if len(args) == 0:
            return self._lib.WindGens_Get_VCutOut()

        # Setter
        (Value,) = args
        self._lib.WindGens_Set_VCutOut(Value)

    def Vss(self, *args):
        """
        Steady state voltage magnitude.
        """
        # Getter
        if len(args) == 0:
            return self._lib.WindGens_Get_Vss()

        # Setter
        (Value,) = args
        self._lib.WindGens_Set_Vss(Value)

    def WindSpeed(self, *args):
        """
        Wind speed in m/s
        """
        # Getter
        if len(args) == 0:
            return self._lib.WindGens_Get_WindSpeed()

        # Setter
        (Value,) = args
        self._lib.WindGens_Set_WindSpeed(Value)

    def XThev(self, *args):
        """
        Per unit Thevenin equivalent reactance (X).
        """
        # Getter
        if len(args) == 0:
            return self._lib.WindGens_Get_XThev()

        # Setter
        (Value,) = args
        self._lib.WindGens_Set_XThev(Value)

    def Phases(self, *args):
        """
        Number of phases

        (API Extension)
        """
        # Getter
        if len(args) == 0:
            return self._lib.WindGens_Get_Phases()

        # Setter
        (Value,) = args
        self._lib.WindGens_Set_Phases(Value)

    def daily(self, *args):
        """
        Name of the loadshape for daily wind speed

        (API Extension)
        """
        # Getter
        if len(args) == 0:
            return self._lib.WindGens_Get_daily()

        # Setter
        (Value,) = args
        self._lib.WindGens_Set_daily(Value)

    def duty(self, *args):
        """
        Name of the loadshape for a duty cycle simulation.

        (API Extension)
        """
        # Getter
        if len(args) == 0:
            return self._lib.WindGens_Get_duty()

        # Setter
        (Value,) = args
        self._lib.WindGens_Set_duty(Value)

    def Yearly(self, *args):
        """
        Name of yearly loadshape

        (API Extension)
        """
        # Getter
        if len(args) == 0:
            return self._lib.WindGens_Get_Yearly()

        # Setter
        (Value,) = args
        self._lib.WindGens_Set_Yearly(Value)

    def IsDelta(self, *args):
        """
        WindGen connection. True/1 if delta connection, False/0 if wye.

        (API Extension)
        """
        # Getter
        if len(args) == 0:
            return self._lib.WindGens_Get_IsDelta()

        # Setter
        (Value,) = args
        self._lib.WindGens_Set_IsDelta(Value)

    def Class(self, *args):
        """
        An arbitrary integer number representing the class of WindGen so that WindGen values may be segregated by class.

        (API Extension)
        """
        # Getter
        if len(args) == 0:
            return self._lib.WindGens_Get_Class_()

        # Setter
        (Value,) = args
        self._lib.WindGens_Set_Class_(Value)

    def Bus1(self, *args):
        """
        Bus to which the WindGen is connected. May include specific node specification.

        (API Extension)
        """
        # Getter
        if len(args) == 0:
            return self._lib.WindGens_Get_Bus1()

        # Setter
        (Value,) = args
        self._lib.WindGens_Set_Bus1(Value)


_WindGens = IWindGens(api_util)

# For backwards compatibility, bind to the default instance
Ag = _WindGens.Ag
AllNames = _WindGens.AllNames
Bus1 = _WindGens.Bus1
Class = _WindGens.Class
Count = _WindGens.Count
Cp = _WindGens.Cp
First = _WindGens.First
Idx = _WindGens.Idx
IsDelta = _WindGens.IsDelta
Lamda = _WindGens.Lamda
NPoles = _WindGens.NPoles
N_WTG = _WindGens.N_WTG
Name = _WindGens.Name
Next = _WindGens.Next
PF = _WindGens.PF
PSS = _WindGens.PSS
Phases = _WindGens.Phases
QFlag = _WindGens.QFlag
QMode = _WindGens.QMode
QSS = _WindGens.QSS
RThev = _WindGens.RThev
Rad = _WindGens.Rad
RegisterNames = _WindGens.RegisterNames
RegisterValues = _WindGens.RegisterValues
VCutIn = _WindGens.VCutIn
VCutOut = _WindGens.VCutOut
Vss = _WindGens.Vss
WindSpeed = _WindGens.WindSpeed
XThev = _WindGens.XThev
Yearly = _WindGens.Yearly
daily = _WindGens.daily
duty = _WindGens.duty
kV = _WindGens.kV
kVA = _WindGens.kVA
kW = _WindGens.kW
kvar = _WindGens.kvar
pd = _WindGens.pd
_columns = _WindGens._columns
__all__ = [
    "Ag",
    "AllNames",
    "Bus1",
    "Class",
    "Count",
    "Cp",
    "First",
    "Idx",
    "IsDelta",
    "Lamda",
    "NPoles",
    "N_WTG",
    "Name",
    "Next",
    "PF",
    "PSS",
    "Phases",
    "QFlag",
    "QMode",
    "QSS",
    "RThev",
    "Rad",
    "RegisterNames",
    "RegisterValues",
    "VCutIn",
    "VCutOut",
    "Vss",
    "WindSpeed",
    "XThev",
    "Yearly",
    "daily",
    "duty",
    "kV",
    "kVA",
    "kW",
    "kvar",
    "pd",
]



from ._utils import api_util
from .Bases import Iterable
from dss import LineUnits


class ILineGeometries(Iterable):
    """
    LineGeometry objects

    **(API Extension)**
    """

    __name__ = "LineGeometries"
    _api_prefix = "LineGeometries"
    __slots__ = []
    _columns = [
        "Name",
        "Idx",
        "Nconds",
        "Phases",
        "RhoEarth",
        "Reduce",
        "Units",
        "Conductors",
        "Xcoords",
        "Ycoords",
        # "Rmatrix",
        # "Xmatrix",
        # "Zmatrix",
        "NormAmps",
        "EmergAmps",
    ]

    def Conductors(self):
        """Array of strings with names of all conductors in the active LineGeometry object"""
        return self._lib.LineGeometries_Get_Conductors()

    def EmergAmps(self, *args):
        """Emergency ampere rating"""
        # Getter
        if len(args) == 0:
            return self._lib.LineGeometries_Get_EmergAmps()

        # Setter
        (Value,) = args
        self._lib.LineGeometries_Set_EmergAmps(Value)

    def NormAmps(self, *args):
        """Normal ampere rating"""
        # Getter
        if len(args) == 0:
            return self._lib.LineGeometries_Get_NormAmps()

        # Setter
        (Value,) = args
        self._lib.LineGeometries_Set_NormAmps(Value)

    def RhoEarth(self, *args):
        # Getter
        if len(args) == 0:
            return self._lib.LineGeometries_Get_RhoEarth()

        # Setter
        (Value,) = args
        self._lib.LineGeometries_Set_RhoEarth(Value)

    def Reduce(self, *args):
        # Getter
        if len(args) == 0:
            return self._lib.LineGeometries_Get_Reduce()

        # Setter
        (Value,) = args
        self._lib.LineGeometries_Set_Reduce(Value)

    def Phases(self, *args):
        """Number of Phases"""
        # Getter
        if len(args) == 0:
            return self._lib.LineGeometries_Get_Phases()

        # Setter
        (Value,) = args
        self._lib.LineGeometries_Set_Phases(Value)

    def Rmatrix(self, Frequency, Length, Units):
        """Resistance matrix, ohms"""
        return self._lib.LineGeometries_Get_Rmatrix_GR(Frequency, Length, Units)

    def Xmatrix(self, Frequency, Length, Units):
        """Reactance matrix, ohms"""
        return self._lib.LineGeometries_Get_Xmatrix_GR(Frequency, Length, Units)

    def Zmatrix(self, Frequency, Length, Units):
        """Complex impedance matrix, ohms"""
        return self._lib.LineGeometries_Get_Zmatrix_GR(Frequency, Length, Units)

    def Cmatrix(self, Frequency, Length, Units):
        """Capacitance matrix, nF"""
        return self._lib.LineGeometries_Get_Cmatrix_GR(Frequency, Length, Units)

    def Units(self, *args):
        # Getter
        if len(args) == 0:
            return [LineUnits(unit) for unit in self._lib.LineGeometries_Get_Units_GR()]

        # Setter
        (Value,) = args
        Value, ValuePtr, ValueCount = self._prepare_int32_array(Value)
        self._lib.LineGeometries_Set_Units(ValuePtr, ValueCount)

    def Xcoords(self, *args):
        """Get/Set the X (horizontal) coordinates of the conductors"""
        # Getter
        if len(args) == 0:
            return self._lib.LineGeometries_Get_Xcoords_GR()

        # Setter
        (Value,) = args
        Value, ValuePtr, ValueCount = self._prepare_float64_array(Value)
        self._lib.LineGeometries_Set_Xcoords(ValuePtr, ValueCount)

    def Ycoords(self, *args):
        """Get/Set the Y (vertical/height) coordinates of the conductors"""
        # Getter
        if len(args) == 0:
            return self._lib.LineGeometries_Get_Ycoords_GR()

        # Setter
        (Value,) = args
        Value, ValuePtr, ValueCount = self._prepare_float64_array(Value)
        self._lib.LineGeometries_Set_Ycoords(ValuePtr, ValueCount)

    def Nconds(self, *args):
        """Number of conductors in this geometry. Default is 3. Triggers memory allocations. Define first!"""
        # Getter
        if len(args) == 0:
            return self._lib.LineGeometries_Get_Nconds()

        # Setter
        (Value,) = args
        self._lib.LineGeometries_Set_Nconds(Value)


_LineGeometries = ILineGeometries(api_util)

# For backwards compatibility, bind to the default instance
AllNames = _LineGeometries.AllNames
Cmatrix = _LineGeometries.Cmatrix
Conductors = _LineGeometries.Conductors
Count = _LineGeometries.Count
EmergAmps = _LineGeometries.EmergAmps
First = _LineGeometries.First
Idx = _LineGeometries.Idx
Name = _LineGeometries.Name
Nconds = _LineGeometries.Nconds
Next = _LineGeometries.Next
NormAmps = _LineGeometries.NormAmps
Phases = _LineGeometries.Phases
Reduce = _LineGeometries.Reduce
RhoEarth = _LineGeometries.RhoEarth
Rmatrix = _LineGeometries.Rmatrix
Units = _LineGeometries.Units
Xcoords = _LineGeometries.Xcoords
Xmatrix = _LineGeometries.Xmatrix
Ycoords = _LineGeometries.Ycoords
Zmatrix = _LineGeometries.Zmatrix
_columns = _LineGeometries._columns
__all__ = [
    "AllNames",
    "Cmatrix",
    "Conductors",
    "Count",
    "EmergAmps",
    "First",
    "Idx",
    "Name",
    "Nconds",
    "Next",
    "NormAmps",
    "Phases",
    "Reduce",
    "RhoEarth",
    "Rmatrix",
    "Units",
    "Xcoords",
    "Xmatrix",
    "Ycoords",
    "Zmatrix",
]



from typing import Union, List
import json
from dss import DSSSaveFlags
from ._utils import api_util
from .Bases import Base

class ICircuit(Base):
    __slots__ = []

    __name__ = "Circuit"
    _api_prefix = "Circuit"
    _columns = [
        "Name",
        "NumBuses",
        "NumNodes",
        "NumCktElements",
        "AllBusDistances",
        "AllBusNames",
        "AllBusVMag",
        "AllBusMagPu",
        "AllBusVolts",
        "AllNodeNames",
        "AllNodeDistances",
        "AllElementNames",
        "YNodeOrder",
        "YNodeVArray",
        "YCurrents",
        "AllElementLosses",
        "LineLosses",
        "Losses",
        "SubstationLosses",
        "TotalPower",
    ]

    def Capacity(self, Start, Increment):
        """
        Compute the maximum load the active circuit can serve in the PRESENT YEAR.

        This method uses the EnergyMeter objects with the registers set with the
        `SET UEREGS= (...)` command for the AutoAdd functions.

        Returns the metered kW (load + losses - generation) and per unit load multiplier
        for the loading level at which something in the system reports an overload or
        undervoltage. If no violations, then it returns the metered kW for peak load
        for the year (1.0 multiplier).

        Aborts and returns 0 if no EnergyMeters.

        Original COM help: https://opendss.epri.com/Capacity1.html
        """
        return self._lib.Circuit_Capacity(Start, Increment)

    def Disable(self, Name):
        """
        Disable a circuit element by name (removes from circuit but leave in database).

        Original COM help: https://opendss.epri.com/Disable.html
        """
        self._lib.Circuit_Disable(Name)

    def Enable(self, Name):
        """
        Enable a circuit element by name

        Original COM help: https://opendss.epri.com/Enable.html
        """
        self._lib.Circuit_Enable(Name)

    def EndOfTimeStepUpdate(self):
        """
        Call `EndOfTimeStepCleanup` in SolutionAlgs (Do cleanup, sample monitors, and increment time).

        Original COM help: https://opendss.epri.com/EndOfTimeStepUpdate.html
        """
        self._lib.Circuit_EndOfTimeStepUpdate()

    def FirstElement(self):
        """
        Set the first element of active class to be the Active element in the active circuit.

        Returns 0 if none.

        Original COM help: https://opendss.epri.com/FirstElement.html
        """
        return self._lib.Circuit_FirstElement()

    def FirstPCElement(self):
        """
        Set the first Power Conversion (PC) element to be the active element.

        Returns 0 if none.

        Original COM help: https://opendss.epri.com/FirstPCElement.html
        """
        return self._lib.Circuit_FirstPCElement()

    def FirstPDElement(self):
        """
        Set the first Power Delivery (PD) element to be the active element.

        Returns 0 if none.

        Original COM help: https://opendss.epri.com/FirstPDElement.html
        """
        return self._lib.Circuit_FirstPDElement()

    def AllNodeDistancesByPhase(self, Phase):
        """Returns an array of doubles representing the distances to parent EnergyMeter. Sequence of array corresponds to other node ByPhase properties."""
        return self._lib.Circuit_Get_AllNodeDistancesByPhase_GR(Phase)

    def AllNodeNamesByPhase(self, Phase):
        """Return array of strings of the node names for the By Phase criteria. Sequence corresponds to other ByPhase properties."""
        return self._lib.Circuit_Get_AllNodeNamesByPhase(Phase)

    def AllNodeVmagByPhase(self, Phase):
        """Returns Array of doubles represent voltage magnitudes for nodes on the specified phase."""
        return self._lib.Circuit_Get_AllNodeVmagByPhase_GR(Phase)

    def AllNodeVmagPUByPhase(self, Phase):
        """Returns array of per unit voltage magnitudes for each node by phase"""
        return self._lib.Circuit_Get_AllNodeVmagPUByPhase_GR(Phase)

    def NextElement(self):
        """
        Set the next element of the active class to be the active element in the active circuit.
        Returns 0 if no more elements..

        Original COM help: https://opendss.epri.com/NextElement.html
        """
        return self._lib.Circuit_NextElement()

    def NextPCElement(self):
        """
        Get the next Power Conversion (PC) element to be the active element.

        Original COM help: https://opendss.epri.com/NextPCElement.html
        """
        return self._lib.Circuit_NextPCElement()

    def NextPDElement(self):
        """
        Get the next Power Delivery (PD) element to be the active element.

        Original COM help: https://opendss.epri.com/NextPDElement.html
        """
        return self._lib.Circuit_NextPDElement()

    def Sample(self):
        """
        Force all Meters and Monitors to take a sample.

        Original COM help: https://opendss.epri.com/Sample.html
        """
        self._lib.Circuit_Sample()

    def SaveSample(self):
        """
        Force all meters and monitors to save their current buffers.

        Original COM help: https://opendss.epri.com/SaveSample.html
        """
        self._lib.Circuit_SaveSample()

    def SetActiveBus(self, BusName):
        """
        Sets Active bus by name.

        Ignores node list. Returns bus index (zero based) compatible with `AllBusNames` and Buses collection.

        Original COM help: https://opendss.epri.com/SetActiveBus.html
        """
        return self._lib.Circuit_SetActiveBus(BusName)

    def SetActiveBusi(self, BusIndex):
        """
        Set ActiveBus by an integer value.

        0-based index compatible with SetActiveBus return value and AllBusNames indexing.
        Returns 0 if OK.

        Original COM help: https://opendss.epri.com/SetActiveBusi.html
        """
        return self._lib.Circuit_SetActiveBusi(BusIndex)

    def SetActiveClass(self, ClassName):
        """
        Set the active class by name.

        Use FirstElement, NextElement to iterate through the class. Returns -1 if fails.

        Original COM help: https://opendss.epri.com/SetActiveClass.html
        """
        return self._lib.Circuit_SetActiveClass(ClassName)

    def SetActiveElement(self, FullName):
        """
        Set the Active Circuit Element using the full object name (e.g. "generator.g1").

        Returns -1 if not found. Else index to be used in CktElements collection or `AllElementNames`.

        Original COM help: https://opendss.epri.com/SetActiveElement.html
        """
        return self._lib.Circuit_SetActiveElement(FullName)

    def UpdateStorage(self):
        """
        Force an update to all storage classes.

        Typically done after a solution. Done automatically in intrinsic solution modes.

        Original COM help: https://opendss.epri.com/UpdateStorage.html
        """
        self._lib.Circuit_UpdateStorage()

    def AllBusDistances(self):
        """
        Returns distance from each bus to parent EnergyMeter. Corresponds to sequence in AllBusNames.

        Original COM help: https://opendss.epri.com/AllBusDistances.html
        """
        return self._lib.Circuit_Get_AllBusDistances_GR()

    def AllBusNames(self):
        """
        Array of strings containing names of all buses in circuit (see AllNodeNames).

        Original COM help: https://opendss.epri.com/AllBusNames.html
        """
        return self._lib.Circuit_Get_AllBusNames()

    def AllBusVMag(self):
        """
        Array of magnitudes (doubles) of voltages at all buses

        Original COM help: https://opendss.epri.com/AllBusVmag.html
        """
        return self._lib.Circuit_Get_AllBusVmag_GR()

    def AllBusMagPu(self):
        """
        Array of all bus voltages (each node) magnitudes in Per unit

        Original COM help: https://opendss.epri.com/AllBusVmagPu.html
        """
        return self._lib.Circuit_Get_AllBusVmagPu_GR()

    def AllBusVolts(self):
        """
        Complex array of all bus, node voltages from most recent solution

        Original COM help: https://opendss.epri.com/AllBusVolts.html
        """
        return self._lib.Circuit_Get_AllBusVolts_GR()

    def AllElementLosses(self):
        """
        Array of total losses (complex) in each circuit element

        Original COM help: https://opendss.epri.com/AllElementLosses.html
        """
        return self._lib.Circuit_Get_AllElementLosses_GR()

    def AllElementNames(self):
        """
        Array of strings containing Full Name of all elements.

        Original COM help: https://opendss.epri.com/AllElementNames.html
        """
        return self._lib.Circuit_Get_AllElementNames()

    def AllNodeDistances(self):
        """
        Returns an array of distances from parent EnergyMeter for each Node. Corresponds to AllBusVMag sequence.

        Original COM help: https://opendss.epri.com/AllNodeDistances.html
        """
        return self._lib.Circuit_Get_AllNodeDistances_GR()

    def AllNodeNames(self):
        """
        Array of strings containing full name of each node in system in same order as returned by AllBusVolts, etc.

        Original COM help: https://opendss.epri.com/AllNodeNames.html
        """
        return self._lib.Circuit_Get_AllNodeNames()

    def LineLosses(self):
        """
        Complex total line losses in the circuit

        Original COM help: https://opendss.epri.com/LineLosses.html
        """
        return self._lib.Circuit_Get_LineLosses_GR()

    def Losses(self):
        """
        Total losses in active circuit, complex number (two-element array of double).

        Original COM help: https://opendss.epri.com/Losses.html
        """
        return self._lib.Circuit_Get_Losses_GR()

    def Name(self):
        """Name of the active circuit."""
        return self._lib.Circuit_Get_Name()

    def NumBuses(self):
        """
        Total number of Buses in the circuit.

        Original COM help: https://opendss.epri.com/NumBuses.html
        """
        return self._lib.Circuit_Get_NumBuses()

    def NumCktElements(self):
        """
        Number of CktElements in the circuit.

        Original COM help: https://opendss.epri.com/NumCktElements.html
        """
        return self._lib.Circuit_Get_NumCktElements()

    def NumNodes(self):
        """
        Total number of nodes in the circuit.

        Original COM help: https://opendss.epri.com/NumNodes1.html
        """
        return self._lib.Circuit_Get_NumNodes()

    def ParentPDElement(self):
        """
        Sets Parent PD element, if any, to be the active circuit element and returns index>0; Returns 0 if it fails or not applicable.

        Original COM help: https://opendss.epri.com/ParentPDElement.html
        """
        return self._lib.Circuit_Get_ParentPDElement()

    def SubstationLosses(self):
        """
        Complex losses in all transformers designated to substations.

        Original COM help: https://opendss.epri.com/SubstationLosses.html
        """
        return self._lib.Circuit_Get_SubstationLosses_GR()

    def SystemY(self):
        """
        (read-only) System Y matrix (after a solution has been performed).
        This is deprecated as it returns a dense matrix. Only use it for small systems.
        For large-scale systems, prefer YMatrix.GetCompressedYMatrix.

        Original COM help: https://opendss.epri.com/SystemY.html
        """
        return self._lib.Circuit_Get_SystemY_GR()

    def TotalPower(self):
        """
        Total power (complex), kVA delivered to the circuit

        Original COM help: https://opendss.epri.com/TotalPower.html
        """
        return self._lib.Circuit_Get_TotalPower_GR()

    def YCurrents(self):
        """
        Array of doubles containing complex injection currents for the present solution. It is the "I" vector of I=YV

        Original COM help: https://opendss.epri.com/YCurrents.html
        """
        return self._lib.Circuit_Get_YCurrents_GR()

    def YNodeOrder(self):
        """
        Array of strings containing the names of the nodes in the same order as the Y matrix

        Original COM help: https://opendss.epri.com/YNodeOrder.html
        """
        return self._lib.Circuit_Get_YNodeOrder()

    def YNodeVArray(self):
        """
        Complex array of actual node voltages in same order as SystemY matrix.

        Original COM help: https://opendss.epri.com/YNodeVarray.html
        """
        return self._lib.Circuit_Get_YNodeVarray_GR()

    def ElementLosses(self, Value):
        """
        Array of total losses (complex) in a selection of elements.
        Use the element indices (starting at 1) as parameter.

        **(API Extension)**
        """
        Value, ValuePtr, ValueCount = self._prepare_int32_array(Value)
        return self._lib.Circuit_Get_ElementLosses_GR(ValuePtr, ValueCount)

    def ToJSON(self, options=0):
        """
        Returns data for all objects and basic circuit properties as a JSON-encoded string.

        The JSON data is organized using the JSON schema proposed at
        https://github.com/dss-extensions/AltDSS-Schema

        The `options` parameter contains bit-flags to toggle specific features.
        See the enum `DSSJSONFlags` or `Obj_ToJSON` (C-API) for more.

        **(API Extension)**
        """
        return self._lib.Circuit_ToJSON(options)

    def FromJSON(self, data, options=0):
        """
        Replaces the circuit, if any, with the one provided from a JSON-encoded string.
        If a Python dict is provided, `json.dumps(data)` is applied first.

        The expected layout is defined from the JSON schema proposed at
        https://github.com/dss-extensions/AltDSS-Schema

        The `options` parameter contains bit-flags to toggle specific features.
        See the enum `DSSJSONFlags`.

        **(API Extension)**
        """
        if isinstance(data, dict):
            data = json.dumps(data).encode()
        elif not isinstance(data, bytes):
            data = data.encode()
        self._lib.Circuit_FromJSON(data, options)

    def Save(self, dirOrFilePath, saveFlags: Union[DSSSaveFlags, List[DSSSaveFlags]]):
        """
        Equivalent of the "save circuit" DSS command, but allows customization
        through the `saveFlags` argument, which is a set of bit flags.
        See the `DSSSaveFlags` enumeration for available flags:

        - `CalcVoltageBases`: Include the command CalcVoltageBases.
        - `SetVoltageBases`: Include commands to set the voltage bases individually.
        - `IncludeOptions`: Include most of the options (from the Set/Get DSS commands).
        - `IncludeDisabled`: Include disabled circuit elements (and LoadShapes).
        - `ExcludeDefault`: Exclude default DSS items if they are not modified by the user.
        - `SingleFile`: Use a single file instead of a folder for output.
        - `KeepOrder`: Save the circuit elements in the order they were loaded in the active circuit. Guarantees better reproducibility, especially when the system is ill-conditioned. Requires "SingleFile" flag.
        - `ExcludeMeterZones`: Do not export meter zones (as "feeders") separately. Has no effect when using a single file.
        - `IsOpen`: Export commands to open terminals of elements.
        - `ToString`: to the result string. Requires "SingleFile" flag.

        If `SingleFile` is enabled, the path argument (`dirOrFilePath`) is the file path,
        otherwise it is the folder path. For string output, the argument is not used.

        **(API Extension)**
        """
        if not isinstance(dirOrFilePath, bytes):
            dirOrFilePath = dirOrFilePath.encode()

        if isinstance(saveFlags, (list, tuple)):
            mask = 0
            for v in saveFlags:
                mask = mask | v

            saveFlags = mask

        return self._api_util.get_string(self._lib.Circuit_Save(dirOrFilePath, saveFlags))

    def Flatten(self) -> None:
        """
        Flatten the circuit

        Flatten the circuit structures, removing any object of the following types:

        - XfmrCode
        - LineCode
        - LineSpacing
        - LineGeometry
        - WireData
        - CNData
        - TSData

        The general data from those objects is propagated to the referencing Line and Transformer objects,
        and the properties on the latter are updated to remove any references to the removed objects.

        This is useful for some converting the DSS circuit to another format, without requiring the user to handle all 
        the types listed above. This, of course, results in some limitations since a lot of detail is removed. Numerically,
        a normal snapshot or daily solution should be the same before and after the flatten operation.

        Available only on AltDSS.
        
        **(API Extension)**
        """
        self._lib.Circuit_Flatten()



_Circuit = ICircuit(api_util)

# For backwards compatibility, bind to the default instance
AllBusDistances = _Circuit.AllBusDistances
AllBusMagPu = _Circuit.AllBusMagPu
AllBusNames = _Circuit.AllBusNames
AllBusVMag = _Circuit.AllBusVMag
AllBusVolts = _Circuit.AllBusVolts
AllElementLosses = _Circuit.AllElementLosses
AllElementNames = _Circuit.AllElementNames
AllNodeDistances = _Circuit.AllNodeDistances
AllNodeDistancesByPhase = _Circuit.AllNodeDistancesByPhase
AllNodeNames = _Circuit.AllNodeNames
AllNodeNamesByPhase = _Circuit.AllNodeNamesByPhase
AllNodeVmagByPhase = _Circuit.AllNodeVmagByPhase
AllNodeVmagPUByPhase = _Circuit.AllNodeVmagPUByPhase
Capacity = _Circuit.Capacity
Disable = _Circuit.Disable
ElementLosses = _Circuit.ElementLosses
Enable = _Circuit.Enable
EndOfTimeStepUpdate = _Circuit.EndOfTimeStepUpdate
FirstElement = _Circuit.FirstElement
FirstPCElement = _Circuit.FirstPCElement
FirstPDElement = _Circuit.FirstPDElement
Flatten = _Circuit.Flatten
FromJSON = _Circuit.FromJSON
LineLosses = _Circuit.LineLosses
Losses = _Circuit.Losses
Name = _Circuit.Name
NextElement = _Circuit.NextElement
NextPCElement = _Circuit.NextPCElement
NextPDElement = _Circuit.NextPDElement
NumBuses = _Circuit.NumBuses
NumCktElements = _Circuit.NumCktElements
NumNodes = _Circuit.NumNodes
ParentPDElement = _Circuit.ParentPDElement
Sample = _Circuit.Sample
Save = _Circuit.Save
SaveSample = _Circuit.SaveSample
SetActiveBus = _Circuit.SetActiveBus
SetActiveBusi = _Circuit.SetActiveBusi
SetActiveClass = _Circuit.SetActiveClass
SetActiveElement = _Circuit.SetActiveElement
SubstationLosses = _Circuit.SubstationLosses
SystemY = _Circuit.SystemY
ToJSON = _Circuit.ToJSON
TotalPower = _Circuit.TotalPower
UpdateStorage = _Circuit.UpdateStorage
YCurrents = _Circuit.YCurrents
YNodeOrder = _Circuit.YNodeOrder
YNodeVArray = _Circuit.YNodeVArray
_columns = _Circuit._columns
__all__ = [
    "AllBusDistances",
    "AllBusMagPu",
    "AllBusNames",
    "AllBusVMag",
    "AllBusVolts",
    "AllElementLosses",
    "AllElementNames",
    "AllNodeDistances",
    "AllNodeDistancesByPhase",
    "AllNodeNames",
    "AllNodeNamesByPhase",
    "AllNodeVmagByPhase",
    "AllNodeVmagPUByPhase",
    "Capacity",
    "Disable",
    "ElementLosses",
    "Enable",
    "EndOfTimeStepUpdate",
    "FirstElement",
    "FirstPCElement",
    "FirstPDElement",
    "Flatten",
    "FromJSON",
    "LineLosses",
    "Losses",
    "Name",
    "NextElement",
    "NextPCElement",
    "NextPDElement",
    "NumBuses",
    "NumCktElements",
    "NumNodes",
    "ParentPDElement",
    "Sample",
    "Save",
    "SaveSample",
    "SetActiveBus",
    "SetActiveBusi",
    "SetActiveClass",
    "SetActiveElement",
    "SubstationLosses",
    "SystemY",
    "ToJSON",
    "TotalPower",
    "UpdateStorage",
    "YCurrents",
    "YNodeOrder",
    "YNodeVArray",
]



from ._utils import api_util
from .Bases import Iterable


class IPVsystems(Iterable):
    __slots__ = []

    __name__ = "PVsystems"
    _api_prefix = "PVSystems"
    _columns = [
        "Name",
        "Idx",
        "pf",
        "Irradiance",
        "IrradianceNow",
        "Pmpp",
        "RegisterNames",
        "RegisterValues",
        "kVARated",
        "kW",
        "kvar",
        "daily",
        "duty",
        "yearly",
        "Tdaily",
        "Tduty",
        "Tyearly",
        "Sensor",
    ]

    def Irradiance(self, *args):
        """
        Get/set the present value of the Irradiance property in kW/m²

        Original COM help: https://opendss.epri.com/Irradiance.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.PVSystems_Get_Irradiance()

        # Setter
        (Value,) = args
        self._lib.PVSystems_Set_Irradiance(Value)

    def pf(self, *args):
        """
        Get/set the power factor for the active PVSystem

        Original COM help: https://opendss.epri.com/PF2.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.PVSystems_Get_PF()

        # Setter
        (Value,) = args
        self._lib.PVSystems_Set_PF(Value)

    def RegisterNames(self):
        """
        Array of PVSystem energy meter register names

        See also the enum `GeneratorRegisters`.

        Original COM help: https://opendss.epri.com/RegisterNames2.html
        """
        return self._lib.PVSystems_Get_RegisterNames()

    def RegisterValues(self):
        """
        Array of doubles containing values in PVSystem registers.

        Original COM help: https://opendss.epri.com/RegisterValues2.html
        """
        return self._lib.PVSystems_Get_RegisterValues_GR()

    def kVARated(self, *args):
        """
        Get/set Rated kVA of the PVSystem

        Original COM help: https://opendss.epri.com/kVArated1.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.PVSystems_Get_kVArated()

        # Setter
        (Value,) = args
        self._lib.PVSystems_Set_kVArated(Value)

    def kW(self):
        """
        Get kW output

        Original COM help: https://opendss.epri.com/kW2.html
        """
        return self._lib.PVSystems_Get_kW()

    def kvar(self, *args):
        """
        Get/set kvar output value

        Original COM help: https://opendss.epri.com/kvar2.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.PVSystems_Get_kvar()

        # Setter
        (Value,) = args
        self._lib.PVSystems_Set_kvar(Value)

    def daily(self, *args):
        """
        Name of the dispatch shape to use for daily simulations. Must be previously
        defined as a Loadshape object of 24 hrs, typically. In the default dispatch
        mode, the PVSystem element uses this loadshape to trigger State changes.

        **(API Extension)**
        """
        # Getter
        if len(args) == 0:
            return self._lib.PVSystems_Get_daily()

        # Setter
        (Value,) = args
        self._lib.PVSystems_Set_daily(Value)

    def duty(self, *args):
        """
        Name of the load shape to use for duty cycle dispatch simulations such as
        for solar ramp rate studies. Must be previously defined as a Loadshape
        object. Typically would have time intervals of 1-5 seconds.

        **(API Extension)**
        """
        # Getter
        if len(args) == 0:
            return self._lib.PVSystems_Get_duty()

        # Setter
        (Value,) = args
        self._lib.PVSystems_Set_duty(Value)

    def yearly(self, *args):
        """
        Dispatch shape to use for yearly simulations. Must be previously defined
        as a Loadshape object. If this is not specified, the Daily dispatch shape,
        if any, is repeated during Yearly solution modes. In the default dispatch
        mode, the PVSystem element uses this loadshape to trigger State changes.

        **(API Extension)**
        """
        # Getter
        if len(args) == 0:
            return self._lib.PVSystems_Get_yearly()

        # Setter
        (Value,) = args
        self._lib.PVSystems_Set_yearly(Value)

    def Tdaily(self, *args):
        """
        Temperature shape to use for daily simulations. Must be previously defined
        as a TShape object of 24 hrs, typically. The PVSystem element uses this
        TShape to determine the Pmpp from the Pmpp vs T curve. Units must agree
        with the Pmpp vs T curve.

        **(API Extension)**
        """
        # Getter
        if len(args) == 0:
            return self._lib.PVSystems_Get_Tdaily()

        # Setter
        (Value,) = args
        self._lib.PVSystems_Set_Tdaily(Value)

    def Tduty(self, *args):
        """
        Temperature shape to use for duty cycle dispatch simulations such as for
        solar ramp rate studies. Must be previously defined as a TShape object.
        Typically would have time intervals of 1-5 seconds. Designate the number
        of points to solve using the Set Number=xxxx command. If there are fewer
        points in the actual shape, the shape is assumed to repeat. The PVSystem
        model uses this TShape to determine the Pmpp from the Pmpp vs T curve.
        Units must agree with the Pmpp vs T curve.

        **(API Extension)**
        """
        # Getter
        if len(args) == 0:
            return self._lib.PVSystems_Get_Tduty()

        # Setter
        (Value,) = args
        self._lib.PVSystems_Set_Tduty(Value)

    def Tyearly(self, *args):
        """
        Temperature shape to use for yearly simulations. Must be previously defined
        as a TShape object. If this is not specified, the Daily dispatch shape, if
        any, is repeated during Yearly solution modes. The PVSystem element uses
        this TShape to determine the Pmpp from the Pmpp vs T curve. Units must
        agree with the Pmpp vs T curve.

        **(API Extension)**
        """
        # Getter
        if len(args) == 0:
            return self._lib.PVSystems_Get_Tyearly()

        # Setter
        (Value,) = args
        self._lib.PVSystems_Set_Tyearly(Value)

    def IrradianceNow(self):
        """
        Returns the current irradiance value for the active PVSystem. Use it to
        know what's the current irradiance value for the PV during a simulation.

        Original COM help: https://opendss.epri.com/IrradianceNow.html
        """
        return self._lib.PVSystems_Get_IrradianceNow()

    def Pmpp(self, *args):
        """
        Gets/sets the rated max power of the PV array for 1.0 kW/m² irradiance
        and a user-selected array temperature of the active PVSystem.

        Original COM help: https://opendss.epri.com/Pmpp.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.PVSystems_Get_Pmpp()

        # Setter
        (Value,) = args
        self._lib.PVSystems_Set_Pmpp(Value)

    def Sensor(self):
        """
        Name of the sensor monitoring this element.

        Original COM help: https://opendss.epri.com/Sensor1.html
        """
        return self._lib.PVSystems_Get_Sensor()


_PVsystems = IPVsystems(api_util)

# For backwards compatibility, bind to the default instance
AllNames = _PVsystems.AllNames
Count = _PVsystems.Count
First = _PVsystems.First
Idx = _PVsystems.Idx
Irradiance = _PVsystems.Irradiance
IrradianceNow = _PVsystems.IrradianceNow
Name = _PVsystems.Name
Next = _PVsystems.Next
Pmpp = _PVsystems.Pmpp
RegisterNames = _PVsystems.RegisterNames
RegisterValues = _PVsystems.RegisterValues
Sensor = _PVsystems.Sensor
Tdaily = _PVsystems.Tdaily
Tduty = _PVsystems.Tduty
Tyearly = _PVsystems.Tyearly
daily = _PVsystems.daily
duty = _PVsystems.duty
kVARated = _PVsystems.kVARated
kW = _PVsystems.kW
kvar = _PVsystems.kvar
pf = _PVsystems.pf
yearly = _PVsystems.yearly
_columns = _PVsystems._columns
__all__ = [
    "AllNames",
    "Count",
    "First",
    "Idx",
    "Irradiance",
    "IrradianceNow",
    "Name",
    "Next",
    "Pmpp",
    "RegisterNames",
    "RegisterValues",
    "Sensor",
    "Tdaily",
    "Tduty",
    "Tyearly",
    "daily",
    "duty",
    "kVARated",
    "kW",
    "kvar",
    "pf",
    "yearly",
]



from ._utils import api_util
from .Bases import Base


class IReduceCkt(Base):
    """Circuit Reduction interface"""

    __slots__ = []

    __name__ = "ReduceCkt"
    _api_prefix = "ReduceCkt"
    _columns = []


    def Zmag(self, *args):
        """
        Zmag (ohms) for Reduce Option for Z of short lines

        Original COM help: https://opendss.epri.com/Zmag.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.ReduceCkt_Get_Zmag()

        # Setter
        (Value,) = args
        self._lib.ReduceCkt_Set_Zmag(Value)

    def KeepLoad(self, *args):
        """
        Keep load flag for Reduction options that remove branches

        Original COM help: https://opendss.epri.com/KeepLoad.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.ReduceCkt_Get_KeepLoad()

        # Setter
        (Value,) = args
        self._lib.ReduceCkt_Set_KeepLoad(Value)

    def EditString(self, *args):
        """
        Edit String for RemoveBranches functions

        Original COM help: https://opendss.epri.com/EditString.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.ReduceCkt_Get_EditString()

        # Setter
        (Value,) = args
        self._lib.ReduceCkt_Set_EditString(Value)

    def StartPDElement(self, *args):
        """
        Start element for Remove Branch function

        Original COM help: https://opendss.epri.com/StartPDElement.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.ReduceCkt_Get_StartPDElement()

        # Setter
        (Value,) = args
        self._lib.ReduceCkt_Set_StartPDElement(Value)

    def EnergyMeter(self, *args):
        """
        Name of EnergyMeter to use for reduction

        Original COM help: https://opendss.epri.com/EnergyMeter1.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.ReduceCkt_Get_EnergyMeter()

        # Setter
        (Value,) = args
        self._lib.ReduceCkt_Set_EnergyMeter(Value)

    def SaveCircuit(self, CktName):
        """
        Save present (reduced) circuit
        Filename is listed in the Text Result interface
        """
        self._lib.ReduceCkt_SaveCircuit(CktName)

    def DoDefault(self):
        """
        Do Default Reduction algorithm

        Original COM help: https://opendss.epri.com/DoDefault.html
        """
        self._lib.ReduceCkt_DoDefault()

    def DoShortLines(self):
        """
        Do ShortLines algorithm: Set Zmag first if you don't want the default

        Original COM help: https://opendss.epri.com/DoShortLines.html
        """
        self._lib.ReduceCkt_DoShortLines()

    def DoDangling(self):
        """
        Reduce Dangling Algorithm; branches with nothing connected

        Original COM help: https://opendss.epri.com/DoDangling.html
        """
        self._lib.ReduceCkt_DoDangling()

    def DoLoopBreak(self):
        """
        Break (disable) all the loops found in the active circuit.

        Disables one of the Line objects at the head of a loop to force the circuit to be radial.
        """
        self._lib.ReduceCkt_DoLoopBreak()

    def DoParallelLines(self):
        """
        Merge all parallel lines found in the circuit to facilitate its reduction.
        """
        self._lib.ReduceCkt_DoParallelLines()

    def DoSwitches(self):
        """
        Merge Line objects in which the IsSwitch property is true with the down-line Line object.
        """
        self._lib.ReduceCkt_DoSwitches()

    def Do1phLaterals(self):
        """
        Remove all 1-phase laterals in the active EnergyMeter's zone.

        Loads and other shunt elements are moved to the parent 3-phase bus.
        """
        self._lib.ReduceCkt_Do1phLaterals()

    def DoBranchRemove(self):
        """
        Remove (disable) all branches down-line from the active PDElement.

        Circuit must have an EnergyMeter on this branch.
        If KeepLoad=Y (default), a new Load element is defined and kW, kvar are set to present power flow solution for the first element eliminated.
        The EditString is applied to each new Load element defined.
        """
        self._lib.ReduceCkt_DoBranchRemove()


_ReduceCkt = IReduceCkt(api_util)

# For backwards compatibility, bind to the default instance
Do1phLaterals = _ReduceCkt.Do1phLaterals
DoBranchRemove = _ReduceCkt.DoBranchRemove
DoDangling = _ReduceCkt.DoDangling
DoDefault = _ReduceCkt.DoDefault
DoLoopBreak = _ReduceCkt.DoLoopBreak
DoParallelLines = _ReduceCkt.DoParallelLines
DoShortLines = _ReduceCkt.DoShortLines
DoSwitches = _ReduceCkt.DoSwitches
EditString = _ReduceCkt.EditString
EnergyMeter = _ReduceCkt.EnergyMeter
KeepLoad = _ReduceCkt.KeepLoad
SaveCircuit = _ReduceCkt.SaveCircuit
StartPDElement = _ReduceCkt.StartPDElement
Zmag = _ReduceCkt.Zmag
_columns = _ReduceCkt._columns
__all__ = [
    "Do1phLaterals",
    "DoBranchRemove",
    "DoDangling",
    "DoDefault",
    "DoLoopBreak",
    "DoParallelLines",
    "DoShortLines",
    "DoSwitches",
    "EditString",
    "EnergyMeter",
    "KeepLoad",
    "SaveCircuit",
    "StartPDElement",
    "Zmag",
]



from ._utils import api_util
from .Bases import Iterable


class IXYCurves(Iterable):
    __slots__ = []

    __name__ = "XYCurves"
    _api_prefix = "XYCurves"
    _columns = [
        "Name",
        "Idx",
        "Npts",
        "XArray",
        "XScale",
        "XShift",
        "YArray",
        "YScale",
        "YShift",
        "X",
        "Y",
    ]

    def Npts(self, *args):
        """
        Get/Set Number of points in X-Y curve

        Original COM help: https://opendss.epri.com/Npts1.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.XYCurves_Get_Npts()

        # Setter
        (Value,) = args
        self._lib.XYCurves_Set_Npts(Value)

    def XArray(self, *args):
        """
        Get/set X values as an array of doubles. When setting, remember to set Npts to max number expected values.

        Original COM help: https://opendss.epri.com/Xarray.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.XYCurves_Get_Xarray_GR()

        # Setter
        (Value,) = args
        Value, ValuePtr, ValueCount = self._prepare_float64_array(Value)
        self._lib.XYCurves_Set_Xarray(ValuePtr, ValueCount)

    def XScale(self, *args):
        """
        Factor to scale X values from original curve

        Original COM help: https://opendss.epri.com/Xscale.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.XYCurves_Get_Xscale()

        # Setter
        (Value,) = args
        self._lib.XYCurves_Set_Xscale(Value)

    def XShift(self, *args):
        """
        Amount to shift X value from original curve

        Original COM help: https://opendss.epri.com/Xshift.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.XYCurves_Get_Xshift()

        # Setter
        (Value,) = args
        self._lib.XYCurves_Set_Xshift(Value)

    def YArray(self, *args):
        """
        Get/set Y values as an array of doubles. When setting, remember to set Npts to max number expected values.

        Original COM help: https://opendss.epri.com/Yarray.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.XYCurves_Get_Yarray_GR()

        # Setter
        (Value,) = args
        Value, ValuePtr, ValueCount = self._prepare_float64_array(Value)
        self._lib.XYCurves_Set_Yarray(ValuePtr, ValueCount)

    def YScale(self, *args):
        """
        Factor to scale Y values from original curve

        Original COM help: https://opendss.epri.com/Yscale.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.XYCurves_Get_Yscale()

        # Setter
        (Value,) = args
        self._lib.XYCurves_Set_Yscale(Value)

    def YShift(self, *args):
        """
        Amount to shift Y value from original curve

        Original COM help: https://opendss.epri.com/Yshift.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.XYCurves_Get_Yshift()

        # Setter
        (Value,) = args
        self._lib.XYCurves_Set_Yshift(Value)

    def X(self, *args):
        """
        Set X value or get interpolated value after setting Y

        Original COM help: https://opendss.epri.com/x4.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.XYCurves_Get_x()

        # Setter
        (Value,) = args
        self._lib.XYCurves_Set_x(Value)

    def Y(self, *args):
        """
        Set Y value or get interpolated Y value after setting X

        Original COM help: https://opendss.epri.com/y1.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.XYCurves_Get_y()

        # Setter
        (Value,) = args
        self._lib.XYCurves_Set_y(Value)


_XYCurves = IXYCurves(api_util)

# For backwards compatibility, bind to the default instance
AllNames = _XYCurves.AllNames
Count = _XYCurves.Count
First = _XYCurves.First
Idx = _XYCurves.Idx
Name = _XYCurves.Name
Next = _XYCurves.Next
Npts = _XYCurves.Npts
X = _XYCurves.X
XArray = _XYCurves.XArray
XScale = _XYCurves.XScale
XShift = _XYCurves.XShift
Y = _XYCurves.Y
YArray = _XYCurves.YArray
YScale = _XYCurves.YScale
YShift = _XYCurves.YShift
_columns = _XYCurves._columns
__all__ = [
    "AllNames",
    "Count",
    "First",
    "Idx",
    "Name",
    "Next",
    "Npts",
    "X",
    "XArray",
    "XScale",
    "XShift",
    "Y",
    "YArray",
    "YScale",
    "YShift",
]



from ._utils import api_util
from .Bases import Iterable


class IRelays(Iterable):
    __slots__ = []

    __name__ = "Relays"
    _api_prefix = "Relays"
    _columns = [
        "Name",
        "Idx",
        "MonitoredObj",
        "MonitoredTerm",
        "SwitchedObj",
        "SwitchedTerm",
        "State",
        "NormalState",
    ]

    def MonitoredObj(self, *args):
        """
        Full name of object this Relay is monitoring.

        Original COM help: https://opendss.epri.com/MonitoredObj3.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Relays_Get_MonitoredObj()

        # Setter
        (Value,) = args
        self._lib.Relays_Set_MonitoredObj(Value)

    def MonitoredTerm(self, *args):
        """
        Number of terminal of monitored element that this Relay is monitoring.

        Original COM help: https://opendss.epri.com/MonitoredTerm3.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Relays_Get_MonitoredTerm()

        # Setter
        (Value,) = args
        self._lib.Relays_Set_MonitoredTerm(Value)

    def SwitchedObj(self, *args):
        """
        Full name of element that will be switched when relay trips.

        Original COM help: https://opendss.epri.com/SwitchedObj2.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Relays_Get_SwitchedObj()

        # Setter
        (Value,) = args
        self._lib.Relays_Set_SwitchedObj(Value)

    def SwitchedTerm(self, *args):
        """
        Terminal number of the switched object that will be opened when the relay trips.

        Original COM help: https://opendss.epri.com/SwitchedTerm2.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Relays_Get_SwitchedTerm()

        # Setter
        (Value,) = args
        self._lib.Relays_Set_SwitchedTerm(Value)

    def Open(self):
        """
        Open relay's controlled element and lock out the relay.

        Original COM help: https://opendss.epri.com/Open4.html
        """
        self._lib.Relays_Open()

    def Close(self):
        """
        Close the switched object controlled by the relay. Resets relay to first operation.

        Original COM help: https://opendss.epri.com/Close5.html
        """
        self._lib.Relays_Close()

    def Reset(self):
        """
        Reset relay to normal state.
        If open, lock out the relay.
        If closed, resets relay to first operation.
        """
        self._lib.Relays_Reset()

    def State(self, *args):
        """
        Present state of relay.
        If set to open, open relay's controlled element and lock out the relay.
        If set to close, close relay's controlled element and resets relay to first operation.
        """
        # Getter
        if len(args) == 0:
            return self._lib.Relays_Get_State()

        # Setter
        (Value,) = args
        self._lib.Relays_Set_State(Value)

    def NormalState(self, *args):
        """
        Normal state of relay.

        Original COM help: https://opendss.epri.com/NormalState3.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Relays_Get_NormalState()

        # Setter
        (Value,) = args
        self._lib.Relays_Set_NormalState(Value)


_Relays = IRelays(api_util)

# For backwards compatibility, bind to the default instance
AllNames = _Relays.AllNames
Close = _Relays.Close
Count = _Relays.Count
First = _Relays.First
Idx = _Relays.Idx
MonitoredObj = _Relays.MonitoredObj
MonitoredTerm = _Relays.MonitoredTerm
Name = _Relays.Name
Next = _Relays.Next
NormalState = _Relays.NormalState
Open = _Relays.Open
Reset = _Relays.Reset
State = _Relays.State
SwitchedObj = _Relays.SwitchedObj
SwitchedTerm = _Relays.SwitchedTerm
_columns = _Relays._columns
__all__ = [
    "AllNames",
    "Close",
    "Count",
    "First",
    "Idx",
    "MonitoredObj",
    "MonitoredTerm",
    "Name",
    "Next",
    "NormalState",
    "Open",
    "Reset",
    "State",
    "SwitchedObj",
    "SwitchedTerm",
]



from ._utils import api_util
from .Bases import Base


class IActiveClass(Base):
    __slots__ = []
    __name__ = "ActiveClass"
    _api_prefix = "ActiveClass"
    _columns = ["ActiveClassName", "ActiveClassParent", "Name", "NumElements"]

    def ActiveClassName(self):
        """
        Returns name of active class.

        Original COM help: https://opendss.epri.com/ActiveClassName.html
        """
        return self._lib.ActiveClass_Get_ActiveClassName()

    def AllNames(self):
        """
        Array of strings consisting of all element names in the active class.

        Original COM help: https://opendss.epri.com/AllNames.html
        """
        return self._lib.ActiveClass_Get_AllNames()

    def Count(self):
        """
        Number of elements in Active Class. Same as NumElements Property.

        Original COM help: https://opendss.epri.com/Count.html
        """
        return self._lib.ActiveClass_Get_Count()

    def First(self):
        """
        Sets first element in the active class to be the active DSS object.
        If the object is a CktElement, ActiveCktElement also points to this element.

        Returns 0 if none.

        Original COM help: https://opendss.epri.com/First.html
        """
        return self._lib.ActiveClass_Get_First()

    def Name(self, *args):
        """
        Name of the Active Element of the Active Class

        Original COM help: https://opendss.epri.com/Name.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.ActiveClass_Get_Name()

        # Setter
        (Value,) = args
        self._lib.ActiveClass_Set_Name(Value)

    def Next(self):
        """
        Sets next element in active class to be the active DSS object.
        If the object is a CktElement, ActiveCktElement also points to this element.

        Returns 0 if no more.

        Original COM help: https://opendss.epri.com/Next.html
        """
        return self._lib.ActiveClass_Get_Next()

    def NumElements(self):
        """
        Number of elements in this class. Same as Count property.

        Original COM help: https://opendss.epri.com/NumElements.html
        """
        return self._lib.ActiveClass_Get_NumElements()

    def ActiveClassParent(self):
        """
        Get the name of the parent class of the active class

        Original COM help: https://opendss.epri.com/ActiveClassParent.html
        """
        return self._lib.ActiveClass_Get_ActiveClassParent()

    def ToJSON(self, options=0):
        """
        Returns the data (as a list) of all elements from the active class as a JSON-encoded string.

        The `options` parameter contains bit-flags to toggle specific features.
        See `Obj_ToJSON` (C-API) for more, or `DSSObj.to_json` in Python.

        Additionally, the `ExcludeDisabled` flag can be used to excluded disabled elements from the output.

        **(API Extension)**
        """
        return self._lib.ActiveClass_ToJSON(options)

    def to_altdss(self):
        """
        Returns a Python object for the current active DSS object in this interface.

        Requires AltDSS-Python.

        *Available only for the AltDSS engine.*

        **(API Extension)**
        """
        ptr = self._lib.ActiveClass_Get_Pointer()
        return self._api_util.get_dss_obj(ptr)


_ActiveClass = IActiveClass(api_util)

# For backwards compatibility, bind to the default instance
ActiveClassName = _ActiveClass.ActiveClassName
ActiveClassParent = _ActiveClass.ActiveClassParent
AllNames = _ActiveClass.AllNames
Count = _ActiveClass.Count
First = _ActiveClass.First
Name = _ActiveClass.Name
Next = _ActiveClass.Next
NumElements = _ActiveClass.NumElements
ToJSON = _ActiveClass.ToJSON
to_altdss = _ActiveClass.to_altdss
_columns = _ActiveClass._columns
__all__ = [
    "ActiveClassName",
    "ActiveClassParent",
    "AllNames",
    "Count",
    "First",
    "Name",
    "Next",
    "NumElements",
    "ToJSON",
    "to_altdss",
]



from ._utils import api_util
from .Bases import Iterable


class ITSData(Iterable):
    """
    TSData objects

    *Not available when using EPRI's OpenDSS distribution.*

    **(API Extension)**
    """

    __name__ = "TSData"
    _api_prefix = "TSData"
    __slots__ = []
    _columns = [
        "Name",
        "Idx",
        "NormAmps",
        "EmergAmps",
        "Rdc",
        "Rac",
        "GMRac",
        "GMRUnits",
        "Radius",
        "RadiusUnits",
        "ResistanceUnits",
        "Diameter",
        "TapeLayer",
        "TapeLap",
        "DiaShield",
        "DiaCable",
        "DiaIns",
        "InsLayer",
        "EpsR",
    ]

    def EmergAmps(self, *args):
        """Emergency ampere rating"""
        # Getter
        if len(args) == 0:
            return self._lib.TSData_Get_EmergAmps()

        # Setter
        (Value,) = args
        self._lib.TSData_Set_EmergAmps(Value)

    def NormAmps(self, *args):
        """Normal Ampere rating"""
        # Getter
        if len(args) == 0:
            return self._lib.TSData_Get_NormAmps()

        # Setter
        (Value,) = args
        self._lib.TSData_Set_NormAmps(Value)

    def Rdc(self, *args):
        # Getter
        if len(args) == 0:
            return self._lib.TSData_Get_Rdc()

        # Setter
        (Value,) = args
        self._lib.TSData_Set_Rdc(Value)

    def Rac(self, *args):
        # Getter
        if len(args) == 0:
            return self._lib.TSData_Get_Rac()

        # Setter
        (Value,) = args
        self._lib.TSData_Set_Rac(Value)

    def GMRac(self, *args):
        # Getter
        if len(args) == 0:
            return self._lib.TSData_Get_GMRac()

        # Setter
        (Value,) = args
        self._lib.TSData_Set_GMRac(Value)

    def GMRUnits(self, *args):
        # Getter
        if len(args) == 0:
            return self._lib.TSData_Get_GMRUnits()

        # Setter
        (Value,) = args
        self._lib.TSData_Set_GMRUnits(Value)

    def Radius(self, *args):
        # Getter
        if len(args) == 0:
            return self._lib.TSData_Get_Radius()

        # Setter
        (Value,) = args
        self._lib.TSData_Set_Radius(Value)

    def RadiusUnits(self, *args):
        # Getter
        if len(args) == 0:
            return self._lib.TSData_Get_RadiusUnits()

        # Setter
        (Value,) = args
        self._lib.TSData_Set_RadiusUnits(Value)

    def ResistanceUnits(self, *args):
        # Getter
        if len(args) == 0:
            return self._lib.TSData_Get_ResistanceUnits()

        # Setter
        (Value,) = args
        self._lib.TSData_Set_ResistanceUnits(Value)

    def Diameter(self, *args):
        # Getter
        if len(args) == 0:
            return self._lib.TSData_Get_Diameter()

        # Setter
        (Value,) = args
        self._lib.TSData_Set_Diameter(Value)

    def EpsR(self, *args):
        # Getter
        if len(args) == 0:
            return self._lib.TSData_Get_EpsR()

        # Setter
        (Value,) = args
        self._lib.TSData_Set_EpsR(Value)

    def InsLayer(self, *args):
        # Getter
        if len(args) == 0:
            return self._lib.TSData_Get_InsLayer()

        # Setter
        (Value,) = args
        self._lib.TSData_Set_InsLayer(Value)

    def DiaIns(self, *args):
        # Getter
        if len(args) == 0:
            return self._lib.TSData_Get_DiaIns()

        # Setter
        (Value,) = args
        self._lib.TSData_Set_DiaIns(Value)

    def DiaCable(self, *args):
        # Getter
        if len(args) == 0:
            return self._lib.TSData_Get_DiaCable()

        # Setter
        (Value,) = args
        self._lib.TSData_Set_DiaCable(Value)

    def DiaShield(self, *args):
        # Getter
        if len(args) == 0:
            return self._lib.TSData_Get_DiaShield()

        # Setter
        (Value,) = args
        self._lib.TSData_Set_DiaShield(Value)

    def TapeLayer(self, *args):
        # Getter
        if len(args) == 0:
            return self._lib.TSData_Get_TapeLayer()

        # Setter
        (Value,) = args
        self._lib.TSData_Set_TapeLayer(Value)

    def TapeLap(self, *args):
        # Getter
        if len(args) == 0:
            return self._lib.TSData_Get_TapeLap()

        # Setter
        (Value,) = args
        self._lib.TSData_Set_TapeLap(Value)


_TSData = ITSData(api_util)

# For backwards compatibility, bind to the default instance
AllNames = _TSData.AllNames
Count = _TSData.Count
DiaCable = _TSData.DiaCable
DiaIns = _TSData.DiaIns
DiaShield = _TSData.DiaShield
Diameter = _TSData.Diameter
EmergAmps = _TSData.EmergAmps
EpsR = _TSData.EpsR
First = _TSData.First
GMRUnits = _TSData.GMRUnits
GMRac = _TSData.GMRac
Idx = _TSData.Idx
InsLayer = _TSData.InsLayer
Name = _TSData.Name
Next = _TSData.Next
NormAmps = _TSData.NormAmps
Rac = _TSData.Rac
Radius = _TSData.Radius
RadiusUnits = _TSData.RadiusUnits
Rdc = _TSData.Rdc
ResistanceUnits = _TSData.ResistanceUnits
TapeLap = _TSData.TapeLap
TapeLayer = _TSData.TapeLayer
_columns = _TSData._columns
__all__ = [
    "AllNames",
    "Count",
    "DiaCable",
    "DiaIns",
    "DiaShield",
    "Diameter",
    "EmergAmps",
    "EpsR",
    "First",
    "GMRUnits",
    "GMRac",
    "Idx",
    "InsLayer",
    "Name",
    "Next",
    "NormAmps",
    "Rac",
    "Radius",
    "RadiusUnits",
    "Rdc",
    "ResistanceUnits",
    "TapeLap",
    "TapeLayer",
]



from ._utils import api_util
from .Bases import Base


class IText(Base):
    __slots__ = []

    __name__ = "Text"
    _api_prefix = "Text"
    _columns = []

    def Command(self, *args):
        """
        Input command string for the DSS.

        Original COM help: https://opendss.epri.com/Command1.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Text_Get_Command()

        # Setter
        (Value,) = args
        self._lib.Text_Set_Command(Value)

    def Result(self):
        """
        Result string for the last command.

        Original COM help: https://opendss.epri.com/Result.html
        """
        return self._lib.Text_Get_Result()

    def Commands(self, Value):
        """
        Runs a list of strings or a large string as commands directly in the DSS engine.
        Intermediate results (from Text.Result) are ignored.

        Value can be a list of strings, or a single large string (usually faster).

        **(API Extension)**
        """
        if isinstance(Value, (str, bytes)):
            self._lib.Text_CommandBlock(Value)
        else:
            self._set_string_array(self._lib.Text_CommandArray, Value)


_Text = IText(api_util)

# For backwards compatibility, bind to the default instance
Command = _Text.Command
Commands = _Text.Commands
Result = _Text.Result
_columns = _Text._columns
__all__ = ["Command", "Commands", "Result"]



from ._utils import api_util
from .Bases import Base


class IParallel(Base):
    """
    Parallel machine interface
    
    On DSS-Extensions, prefer using DSSContexts and native threading capabilities of your programming
    language, if available.
    """

    __slots__ = []

    __name__ = "Parallel"
    _api_prefix = "Parallel"
    _columns = []

    def CreateActor(self):
        """
        Create a new actor, if there are still cores available.
        """
        self._lib.Parallel_CreateActor()

    def Wait(self):
        """
        Suspends the host's thread until all the OpenDSS running jobs finish.

        Original COM help: https://opendss.epri.com/Wait.html
        """
        self._lib.Parallel_Wait()

    def ActiveActor(self, *args):
        """
        Gets/sets the ID of the Active Actor

        Original COM help: https://opendss.epri.com/ActiveActor.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Parallel_Get_ActiveActor()

        # Setter
        (Value,) = args
        self._lib.Parallel_Set_ActiveActor(Value)

    def ActiveParallel(self, *args):
        """
        (read) Sets ON/OFF (1/0) Parallel features of the Engine
        (write) Delivers if the Parallel features of the Engine are Active

        Original COM help: https://opendss.epri.com/ActiveParallel.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Parallel_Get_ActiveParallel()

        # Setter
        (Value,) = args
        self._lib.Parallel_Set_ActiveParallel(Value)

    def ActorCPU(self, *args):
        """
        Gets/sets the CPU of the Active Actor

        Original COM help: https://opendss.epri.com/ActorCPU.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Parallel_Get_ActorCPU()

        # Setter
        (Value,) = args
        self._lib.Parallel_Set_ActorCPU(Value)

    def ActorProgress(self):
        """
        Gets the progress of all existing actors in pct

        Original COM help: https://opendss.epri.com/ActorProgress.html
        """
        return self._lib.Parallel_Get_ActorProgress_GR()

    def ActorStatus(self):
        """
        Gets the status of each actor

        Original COM help: https://opendss.epri.com/ActorStatus.html
        """
        return self._lib.Parallel_Get_ActorStatus_GR()

    def ConcatenateReports(self, *args):
        """
        (read) Reads the values of the ConcatenateReports option (1=enabled, 0=disabled)
        (write) Enable/Disable (1/0) the ConcatenateReports option for extracting monitors data

        Original COM help: https://opendss.epri.com/ConcatenateReports.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Parallel_Get_ConcatenateReports()

        # Setter
        (Value,) = args
        self._lib.Parallel_Set_ConcatenateReports(Value)

    def NumCPUs(self):
        """
        Delivers the number of CPUs on the current PC

        Original COM help: https://opendss.epri.com/NumCPUs.html
        """
        return self._lib.Parallel_Get_NumCPUs()

    def NumCores(self):
        """
        Delivers the number of Cores of the local PC

        Original COM help: https://opendss.epri.com/NumCores.html
        """
        return self._lib.Parallel_Get_NumCores()

    def NumOfActors(self):
        """
        Gets the number of Actors created

        Original COM help: https://opendss.epri.com/NumOfActors.html
        """
        return self._lib.Parallel_Get_NumOfActors()


_Parallel = IParallel(api_util)

# For backwards compatibility, bind to the default instance
ActiveActor = _Parallel.ActiveActor
ActiveParallel = _Parallel.ActiveParallel
ActorCPU = _Parallel.ActorCPU
ActorProgress = _Parallel.ActorProgress
ActorStatus = _Parallel.ActorStatus
ConcatenateReports = _Parallel.ConcatenateReports
CreateActor = _Parallel.CreateActor
NumCPUs = _Parallel.NumCPUs
NumCores = _Parallel.NumCores
NumOfActors = _Parallel.NumOfActors
Wait = _Parallel.Wait
_columns = _Parallel._columns
__all__ = [
    "ActiveActor",
    "ActiveParallel",
    "ActorCPU",
    "ActorProgress",
    "ActorStatus",
    "ConcatenateReports",
    "CreateActor",
    "NumCPUs",
    "NumCores",
    "NumOfActors",
    "Wait",
]



from __future__ import annotations
import warnings
from typing import TYPE_CHECKING
from weakref import WeakKeyDictionary
from typing import AnyStr, List, Union
from ._utils import DSSException, dss_py, OPENDSSDIRECT_PY_USE_NUMPY
from ._utils import lib as dss_lib
from ._utils import ffi as dss_ffi
from .Bases import Base
from . import utils as _utils
from .utils import run_command
from .ActiveClass import _ActiveClass, IActiveClass
from .Basic import _Basic, IBasic
from .Bus import _Bus, IBus
from .CapControls import _CapControls, ICapControls
from .Capacitors import _Capacitors, ICapacitors
from .Circuit import _Circuit, ICircuit
from .CktElement import _CktElement, ICktElement
from .CtrlQueue import _CtrlQueue, ICtrlQueue
from .DSSCore import _DSSCore
from .Element import _Element, IElement
from .Error import _Error, IError
from .Executive import _Executive, IExecutive
from .Fuses import _Fuses, IFuses
from .Generators import _Generators, IGenerators
from .GICSources import _GICSources, IGICSources
from .Isource import _Isource, IIsource
from .LineCodes import _LineCodes, ILineCodes
from .Lines import _Lines, ILines
from .LoadShape import _LoadShape, ILoadShape
from .Loads import _Loads, ILoads
from .Meters import _Meters, IMeters
from .Monitors import _Monitors, IMonitors
from .PDElements import _PDElements, IPDElements
from .PVsystems import _PVsystems, IPVsystems
from .Parallel import _Parallel, IParallel
from .Parser import _Parser, IParser
from .Progress import _Progress, IProgress
from .Properties import _Properties, IProperties
from .Reclosers import _Reclosers, IReclosers
from .RegControls import _RegControls, IRegControls
from .Relays import _Relays, IRelays
from .Sensors import _Sensors, ISensors
from .Settings import _Settings, ISettings
from .Solution import _Solution, ISolution
from .Storages import _Storages, IStorages
from .SwtControls import _SwtControls, ISwtControls
from .Text import _Text, IText
from .Topology import _Topology, ITopology
from .Transformers import _Transformers, ITransformers
from .Vsources import _Vsources, IVsources
from .XYCurves import _XYCurves, IXYCurves
from .YMatrix import _YMatrix, IYMatrix
from .CNData import _CNData, ICNData
from .LineGeometries import _LineGeometries, ILineGeometries
from .LineSpacings import _LineSpacings, ILineSpacings
from .Reactors import _Reactors, IReactors
from .ReduceCkt import _ReduceCkt, IReduceCkt
from .TSData import _TSData, ITSData
from .WindGens import _WindGens, IWindGens
from .WireData import _WireData, IWireData
from .ZIP import _ZIP, IZIP

if TYPE_CHECKING:
    # DSS-Python is always required at the moment
    from dss import IDSS as DSSPython

    # AltDSS is optional
    try:
        from altdss import AltDSS
    except:
        pass


class OpenDSSDirect(Base):
    """
    This is the main OpenDSSDirect.py class that wraps all the available interfaces.
    
    Besides the traditional submodules from previous (pre-v0.9) versions, it has a
    shortcut for commands as the call operator, plus some utility methods.
        
    Note that users do not typically need to create instances of this class manually.
    For creating multiple, separate DSS engine instances, use the [`NewContext()`](OpenDSSDirect.NewContext)
    method.
    For Oddie-wrapped libraries (EPRI's OpenDSS and OpenDSS-C), prefer the dedicated constructors and classes
    (e.g. `IOddieDSS`, `EPRIOpenDSSC`, `EPRIOpenDSS`).

    This class also provides function to return DSS-Python and AltDSS representations
    of the same engine, allowing users to mix usage of the packages more easily.
    """
        
    try:
        # Version is now populated by the build script based
        # on the git tags. See pyproject.toml for more info.
        from ._version import __version__
    except:
        __version__ = "0.0dev"

    DSSException = DSSException
    """Shortcut to the `DSSException` class."""

    __slots__ = [
        "ActiveClass",
        "Basic",
        "Bus",
        "CapControls",
        "Capacitors",
        "Circuit",
        "CktElement",
        "CtrlQueue",
        "DSSCore",
        "Element",
        "Error",
        "Executive",
        "Fuses",
        "Generators",
        "GICSources",
        "Isource",
        "LineCodes",
        "Lines",
        "LoadShape",
        "Loads",
        "Meters",
        "Monitors",
        "PDElements",
        "PVsystems",
        "Parallel",
        "Parser",
        "Progress",
        "Properties",
        "Reclosers",
        "RegControls",
        "Relays",
        "Sensors",
        "Settings",
        "Solution",
        "Storages",
        "SwtControls",
        "Text",
        "Topology",
        "Transformers",
        "Vsources",
        "XYCurves",
        "YMatrix",
        "CNData",
        "LineGeometries",
        "LineSpacings",
        "Reactors",
        "ReduceCkt",
        "TSData",
        "WindGens",
        "WireData",
        "ZIP",
        "dss_lib",
        "dss_ffi",
        "dss",
        "Command",
        "Commands",
        "Version",
    ]

    utils = _utils
    """Shortcut to the [`utils`](opendssdirect.utils) module."""

    # `_ctx_to_dss` is to be used in callbacks (mapping the pointer 
    # to the equivalent Python object)
    _ctx_to_dss = WeakKeyDictionary()

    @classmethod
    def _get_instance(cls: OpenDSSDirect, api_util: dss_py.AltDSSAPIUtil = None, ctx=None) -> OpenDSSDirect:
        """
        If there is an existing OpenDSSDirect instance for a context, return it.
        Otherwise, try to wrap the context into a new OpenDSSDirect.py API instance.
        """
        if api_util is None:
            # If none exists, something is probably wrong elsewhere,
            # so let's allow the IndexError to propagate
            api_util = dss_py.AltDSSAPIUtil._ctx_to_util[ctx]

        dss = cls._ctx_to_dss.get(api_util.ctx)
        if dss is None:
            dss = cls(api_util)

        return dss

    def __init__(self, ctx_api_util=None, prefer_lists=None):
        """
        Creates a new OpenDSSDirect.py instance for the DSS context specified in `ctx_api_util`.

        Not intended for typical usage. For creating new separate DSS instances, refer
        to the [`NewContext()`](OpenDSSDirect.NewContext) method.
        """
        
        if ctx_api_util is None:
            # If ctx_api_util is None, grab the default instance, already initialized.
            ctx_api_util = _ActiveClass._api_util

        if ctx_api_util._opendssdirect is None:
            ctx_api_util._opendssdirect = self

        Base.__init__(self, ctx_api_util)
        if prefer_lists is None:
            self._lib.prefer_lists = not OPENDSSDIRECT_PY_USE_NUMPY
        else:
            if self._lib.prefer_lists != prefer_lists:
                warnings.warn('"prefer_lists" is deprecated. Use instead the method `Settings.PreferLists` to toggle the behavior on an existing OpenDSSDirect instance. Current behavior: the value passed in "prefer_lists" will update the value on the new instance and potentially affect existing instances.', DeprecationWarning, stacklevel=2)

            self._lib.prefer_lists = prefer_lists

        OpenDSSDirect._ctx_to_dss[ctx_api_util.ctx] = self
        object.__setattr__(self, '_frozen_attrs', False)
        
        self.dss_lib = ctx_api_util.lib_unpatched
        """
        `dss_lib` provides access to the low-level CFFI library; this is not recommended for most users and may 
        change across versions. The intention is to allow some (very) advanced operations directly, when the
        high-level interface either does not model some specific detail, or when users know how to manually
        address some data directly with lower overhead.

        Note that the prepared alternative, which already abstracts away some details, is available in the 
        `_lib` attribute. That one is private and could change when required to introduce new features. See
        the source code for OpenDSSDirect.py or DSS-Python for examples of how it is used.

        **WARNING**: if used incorrectly, could even crash your computer.
        """


        self.dss_ffi = ctx_api_util.ffi
        self.dss = self

        # Check if we're creating the default instance, with the same options
        grab_default = (ctx_api_util is _ActiveClass._api_util)

        if grab_default:
            self.ActiveClass = _ActiveClass
            self.Basic = _Basic
            self.Bus = _Bus
            self.CapControls = _CapControls
            self.Capacitors = _Capacitors
            self.Circuit = _Circuit
            self.CktElement = _CktElement
            self.CtrlQueue = _CtrlQueue
            self.DSSCore = _DSSCore
            self.Element = _Element
            self.Error = _Error
            self.Executive = _Executive
            self.Fuses = _Fuses
            self.Generators = _Generators
            self.GICSources = _GICSources
            self.Isource = _Isource
            self.LineCodes = _LineCodes
            self.Lines = _Lines
            self.LoadShape = _LoadShape
            self.Loads = _Loads
            self.Meters = _Meters
            self.Monitors = _Monitors
            self.PDElements = _PDElements
            self.PVsystems = _PVsystems
            self.Parallel = _Parallel
            self.Parser = _Parser
            self.Progress = _Progress
            self.Properties = _Properties
            self.Reclosers = _Reclosers
            self.RegControls = _RegControls
            self.Relays = _Relays
            self.Sensors = _Sensors
            self.Settings = _Settings
            self.Solution = _Solution
            self.Storages = _Storages
            self.SwtControls = _SwtControls
            self.Text = _Text
            self.Topology = _Topology
            self.Transformers = _Transformers
            self.Vsources = _Vsources
            self.XYCurves = _XYCurves
            self.YMatrix = _YMatrix
            self.CNData = _CNData
            self.LineGeometries = _LineGeometries
            self.LineSpacings = _LineSpacings
            self.Reactors = _Reactors
            self.ReduceCkt = _ReduceCkt
            self.TSData = _TSData
            self.WindGens = _WindGens
            self.WireData = _WireData
            self.ZIP = _ZIP

            # Shortcuts
            self.Command = self.Text.Command
            self.Commands = self.Text.Commands
            self.Version = self.Basic.Version

            object.__setattr__(self, '_frozen_attrs', True)
            return
        
        not_using_oddie = not ctx_api_util._is_oddie

        self.ActiveClass = IActiveClass(ctx_api_util)
        self.Basic = IBasic(ctx_api_util)
        self.Bus = IBus(ctx_api_util)
        self.CapControls = ICapControls(ctx_api_util)
        self.Capacitors = ICapacitors(ctx_api_util)
        self.Circuit = ICircuit(ctx_api_util)
        self.CktElement = ICktElement(ctx_api_util)
        self.CtrlQueue = ICtrlQueue(ctx_api_util)
        self.Element = IElement(ctx_api_util)
        self.Error = IError(ctx_api_util)
        self.Executive = IExecutive(ctx_api_util)
        self.Fuses = IFuses(ctx_api_util)
        self.Generators = IGenerators(ctx_api_util)
        self.GICSources = IGICSources(ctx_api_util)
        self.Isource = IIsource(ctx_api_util)
        self.LineCodes = ILineCodes(ctx_api_util)
        self.Lines = ILines(ctx_api_util)
        self.LoadShape = ILoadShape(ctx_api_util)
        self.Loads = ILoads(ctx_api_util)
        self.Meters = IMeters(ctx_api_util)
        self.Monitors = IMonitors(ctx_api_util)
        self.PDElements = IPDElements(ctx_api_util)
        self.PVsystems = IPVsystems(ctx_api_util)
        self.Parallel = IParallel(ctx_api_util)
        self.Parser = IParser(ctx_api_util)
        self.Progress = IProgress(ctx_api_util)
        self.Properties = IProperties(ctx_api_util)
        self.Reclosers = IReclosers(ctx_api_util)
        self.RegControls = IRegControls(ctx_api_util)
        self.Relays = IRelays(ctx_api_util)
        self.Sensors = ISensors(ctx_api_util)
        self.Settings = ISettings(ctx_api_util)
        self.Solution = ISolution(ctx_api_util)
        self.Storages = IStorages(ctx_api_util)
        self.SwtControls = ISwtControls(ctx_api_util)
        self.Text = IText(ctx_api_util)
        self.Topology = ITopology(ctx_api_util)
        self.Transformers = ITransformers(ctx_api_util)
        self.Vsources = IVsources(ctx_api_util)
        self.XYCurves = IXYCurves(ctx_api_util)
        self.YMatrix = IYMatrix(ctx_api_util)
        self.CNData = ICNData(ctx_api_util) if not_using_oddie else None
        self.LineGeometries = ILineGeometries(ctx_api_util) if not_using_oddie else None
        self.LineSpacings = ILineSpacings(ctx_api_util) if not_using_oddie else None
        self.Reactors = IReactors(ctx_api_util)
        self.ReduceCkt = IReduceCkt(ctx_api_util)
        self.TSData = ITSData(ctx_api_util) if not_using_oddie else None
        self.WindGens = IWindGens(ctx_api_util)
        self.WireData = IWireData(ctx_api_util) if not_using_oddie else None
        self.ZIP = IZIP(ctx_api_util)
        self.DSSCore = self.YMatrix

        # Shortcuts
        self.Command = self.Text.Command
        self.Commands = self.Text.Commands
        self.Version = self.Basic.Version

        object.__setattr__(self, '_frozen_attrs', True)


    def run_command(self, text):
        """
        Use the Text interface of OpenDSS, grabbing all output text in a string.
        
        This is **deprecated** since it doesn't handle errors as exceptions and can confuse users.

        Use instead: [`dss("commands")`](#opendssdirect.OpenDSSDirect.OpenDSSDirect.__call__), [`dss.Text.Command("command")`](#opendssdirect.Text.IText.Command), or [`dss.Text.Commands("commands")`](#opendssdirect.Text.IText.Commands).
        """
        run_command(text, self)


    def NewCircuit(self, name: str):
        """
        Make a new circuit.

        Original COM help: https://opendss.epri.com/NewCircuit.html
        """
        self._lib.DSS_NewCircuit(name)
        return "New Circuit" # self.ActiveCircuit


    def NewContext(self):
        """
        Creates a new DSS engine context, wrapped by the OpenDSSDirect.py classes.

        A DSS Context encapsulates most of the global state of the original OpenDSS engine,
        allowing the user to create multiple instances in the same process. By creating contexts
        manually, the management of threads and potential issues should be handled by the user.

        **(API Extension)**
        """
        from .OpenDSSDirect import OpenDSSDirect
        ffi = self._api_util.ffi
        lib = self._api_util.lib_unpatched
        new_ctx = ffi.gc(lib.ctx_New(), lib.ctx_Dispose)
        new_api_util = dss_py.AltDSSAPIUtil(ffi, lib, new_ctx, parent=self._api_util)
        return OpenDSSDirect(new_api_util, prefer_lists=self._lib.prefer_lists)

    def to_dss_python(self) -> DSSPython:
        """
        Returns an instance of DSS-Python for the active DSS Context.
        """
        from dss import IDSS as DSSPython
        return DSSPython._get_instance(ctx=self._api_util.ctx, api_util=self._api_util)

    def to_altdss(self) -> AltDSS:
        """
        Returns an instance of AltDSS for the active DSS Context.

        A compatible AltDSS (`pip install altdss`) is required.
        """
        from altdss import AltDSS
        return AltDSS._get_instance(ctx=self._api_util.ctx, api_util=self._api_util)

    def is_oddie(self) -> bool:
        """
        Returns True if this instance is based on the Oddie compatibility layer for
        the official OpenDSS Direct API (a.k.a. DCSL).
        
        Note that the default engine in OpenDSSDirect.py has been based on AltDSS since
        2018, even though it was not called AltDSS then.
        """
        return self._api_util._is_oddie

    def __call__(self, cmds: Union[AnyStr, List[AnyStr]] = None):
        '''
        Shortcut to pass text commands. Allow single and multiple commands in a string, or a list of commands.

        For multiple commands, a big string tends to be the faster.

        Equivalent to `OpenDSSDirect.Text.Commands`

        Examples:

        ```python

            # single command
            dss("new Circuit.test") 

            # list of commands (either will work)
            dss(["new Circuit.test", "new Line.line1 bus1=a bus2=b"])

            # block of commands in a big string
            dss("""
                clear
                new Circuit.test
                new Line.line1 bus1=a bus2=b
                new Load.load1 bus1=a bus2=b
            """)
        ```

        **(API Extension)**
        '''
        # self.Commands(cmds) -- inlined
        if isinstance(cmds, (str, bytes)):
            self._lib.Text_CommandBlock(cmds)
        else:
            self._lib.Text_CommandArray(cmds)


    def ShareGeneral(self, otherContext, skip_cmds=None, skip_file_regexp=None):
        """
        Share general DSS objects from this AltDSS (engine) context to another.

        **WARNING:** currently, the pointers are not tracked! The user must ensure this context
        and its objects are kept alive while other contexts require it.

        Optionally, as a shortcut, the user can provide `skip_cmds` to be passed to the `Settings.SkipCommands`
        and `skip_file_regexp` to be passed to `Settings.SkipFileRegExp`, in the second DSS context.

        *Note*: If the `clear` command is included in `Settings.SkipCommands`, the `ClearAll()` Python method 
        can still be called and it will reset both skip settings.

        ***EXPERIMENTAL***

        **(API Extension)**
        """
        if self._api_util._is_oddie or otherContext._api_util._is_oddie:
            raise ValueError("Only AltDSS engine contexts can share data.")
        self._lib.ctx_ShareGeneral(otherContext._api_util.ctx)
        if skip_cmds is not None:
            otherContext.Settings.SkipCommands(skip_cmds)
        if skip_file_regexp is not None:
            otherContext.Settings.SkipFileRegExp(skip_file_regexp)



dss = OpenDSSDirect(None)
ActiveClass = _ActiveClass
Basic = _Basic
Bus = _Bus
CapControls = _CapControls
Capacitors = _Capacitors
Circuit = _Circuit
CktElement = _CktElement
CtrlQueue = _CtrlQueue
DSSCore = _DSSCore
Element = _Element
Error = _Error
Executive = _Executive
Fuses = _Fuses
Generators = _Generators
GICSources = _GICSources
Isource = _Isource
LineCodes = _LineCodes
Lines = _Lines
LoadShape = _LoadShape
Loads = _Loads
Meters = _Meters
Monitors = _Monitors
PDElements = _PDElements
PVsystems = _PVsystems
Parallel = _Parallel
Parser = _Parser
Progress = _Progress
Properties = _Properties
Reclosers = _Reclosers
RegControls = _RegControls
Relays = _Relays
Sensors = _Sensors
Settings = _Settings
Solution = _Solution
Storages = _Storages
SwtControls = _SwtControls
Text = _Text
Topology = _Topology
Transformers = _Transformers
Vsources = _Vsources
XYCurves = _XYCurves
YMatrix = _YMatrix
CNData = _CNData
LineGeometries = _LineGeometries
LineSpacings = _LineSpacings
Reactors = _Reactors
ReduceCkt = _ReduceCkt
TSData = _TSData
WindGens = _WindGens
WireData = _WireData
ZIP = _ZIP
is_oddie = dss.is_oddie
to_altdss = dss.to_altdss
to_dss_python = dss.to_dss_python
NewCircuit = dss.NewCircuit
NewContext = dss.NewContext
ShareGeneral = dss.ShareGeneral

__all__ = ["OpenDSSDirect", "dss", "OPENDSSDIRECT_PY_USE_NUMPY"]



from ._utils import api_util
from .Bases import Base


class ITopology(Base):
    __slots__ = []

    __name__ = "Topology"
    _api_prefix = "Topology"
    _columns = [
        "NumIsolatedLoads",
        "AllIsolatedBranches",
        "NumIsolatedBranches",
        "AllIsolatedLoads",
        "ActiveLevel",
        "BranchName",
        "AllLoopedPairs",
        "NumLoops",
        "ActiveBranch",
    ]

    def ActiveBranch(self):
        """
        Returns index of the active branch

        Original COM help: https://opendss.epri.com/ActiveBranch.html
        """
        return self._lib.Topology_Get_ActiveBranch()

    def ActiveLevel(self):
        """
        Topological depth of the active branch

        Original COM help: https://opendss.epri.com/ActiveLevel.html
        """
        return self._lib.Topology_Get_ActiveLevel()

    def AllIsolatedBranches(self):
        """
        Array of all isolated branch names.

        Original COM help: https://opendss.epri.com/AllIsolatedBranches.html
        """
        return self._lib.Topology_Get_AllIsolatedBranches()

    def AllIsolatedLoads(self):
        """
        Array of all isolated load names.

        Original COM help: https://opendss.epri.com/AllIsolatedLoads.html
        """
        return self._lib.Topology_Get_AllIsolatedLoads()

    def AllLoopedPairs(self):
        """
        Array of all looped element names, by pairs.

        Original COM help: https://opendss.epri.com/AllLoopedPairs.html
        """
        return self._lib.Topology_Get_AllLoopedPairs()

    def BackwardBranch(self):
        """
        Move back toward the source, return index of new active branch, or 0 if no more.

        Original COM help: https://opendss.epri.com/BackwardBranch.html
        """
        return self._lib.Topology_Get_BackwardBranch()

    def BranchName(self, *args):
        """
        Name of the active branch.

        Original COM help: https://opendss.epri.com/BranchName.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Topology_Get_BranchName()

        # Setter
        (Value,) = args
        self._lib.Topology_Set_BranchName(Value)

    def BusName(self, *args):
        """
        Set the active branch to one containing this bus, return index or 0 if not found

        Original COM help: https://opendss.epri.com/BusName.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Topology_Get_BusName()

        # Setter
        (Value,) = args
        self._lib.Topology_Set_BusName(Value)

    def First(self):
        """
        Sets the first branch active, returns 0 if none.

        Original COM help: https://opendss.epri.com/First19.html
        """
        return self._lib.Topology_Get_First()

    def FirstLoad(self):
        """
        First load at the active branch, return index or 0 if none.

        Original COM help: https://opendss.epri.com/FirstLoad.html
        """
        return self._lib.Topology_Get_FirstLoad()

    def ForwardBranch(self):
        """
        Move forward in the tree, return index of new active branch or 0 if no more

        Original COM help: https://opendss.epri.com/ForwardBranch.html
        """
        return self._lib.Topology_Get_ForwardBranch()

    def LoopedBranch(self):
        """
        Move to looped branch, return index or 0 if none.

        Original COM help: https://opendss.epri.com/LoopedBranch.html
        """
        return self._lib.Topology_Get_LoopedBranch()

    def Next(self):
        """
        Sets the next branch active, returns 0 if no more.

        Original COM help: https://opendss.epri.com/Next18.html
        """
        return self._lib.Topology_Get_Next()

    def NextLoad(self):
        """
        Next load at the active branch, return index or 0 if no more.

        Original COM help: https://opendss.epri.com/NextLoad.html
        """
        return self._lib.Topology_Get_NextLoad()

    def NumIsolatedBranches(self):
        """
        Number of isolated branches (PD elements and capacitors).

        Original COM help: https://opendss.epri.com/NumIsolatedBranches.html
        """
        return self._lib.Topology_Get_NumIsolatedBranches()

    def NumIsolatedLoads(self):
        """
        Number of isolated loads

        Original COM help: https://opendss.epri.com/NumIsolatedLoads.html
        """
        return self._lib.Topology_Get_NumIsolatedLoads()

    def NumLoops(self):
        """
        Number of loops

        Original COM help: https://opendss.epri.com/NumLoops.html
        """
        return self._lib.Topology_Get_NumLoops()

    def ParallelBranch(self):
        """
        Move to directly parallel branch, return index or 0 if none.

        Original COM help: https://opendss.epri.com/ParallelBranch.html
        """
        return self._lib.Topology_Get_ParallelBranch()


_Topology = ITopology(api_util)

# For backwards compatibility, bind to the default instance
ActiveBranch = _Topology.ActiveBranch
ActiveLevel = _Topology.ActiveLevel
AllIsolatedBranches = _Topology.AllIsolatedBranches
AllIsolatedLoads = _Topology.AllIsolatedLoads
AllLoopedPairs = _Topology.AllLoopedPairs
BackwardBranch = _Topology.BackwardBranch
BranchName = _Topology.BranchName
BusName = _Topology.BusName
First = _Topology.First
FirstLoad = _Topology.FirstLoad
ForwardBranch = _Topology.ForwardBranch
LoopedBranch = _Topology.LoopedBranch
Next = _Topology.Next
NextLoad = _Topology.NextLoad
NumIsolatedBranches = _Topology.NumIsolatedBranches
NumIsolatedLoads = _Topology.NumIsolatedLoads
NumLoops = _Topology.NumLoops
ParallelBranch = _Topology.ParallelBranch
_columns = _Topology._columns
__all__ = [
    "ActiveBranch",
    "ActiveLevel",
    "AllIsolatedBranches",
    "AllIsolatedLoads",
    "AllLoopedPairs",
    "BackwardBranch",
    "BranchName",
    "BusName",
    "First",
    "FirstLoad",
    "ForwardBranch",
    "LoopedBranch",
    "Next",
    "NextLoad",
    "NumIsolatedBranches",
    "NumIsolatedLoads",
    "NumLoops",
    "ParallelBranch",
]



from ._utils import api_util
from .Bases import Iterable


class ICapacitors(Iterable):
    __slots__ = []

    __name__ = "Capacitors"
    _api_prefix = "Capacitors"
    _columns = [
        "Name",
        "Idx",
        "kV",
        "NumSteps",
        "AvailableSteps",
        "IsDelta",
        "States",
        "kvar",
    ]

    def AddStep(self):
        return self._lib.Capacitors_AddStep()

    def Close(self):
        self._lib.Capacitors_Close()

    def Open(self):
        self._lib.Capacitors_Open()

    def SubtractStep(self):
        return self._lib.Capacitors_SubtractStep()

    def AvailableSteps(self):
        """
        Number of Steps available in cap bank to be switched ON.

        Original COM help: https://opendss.epri.com/AvailableSteps.html
        """
        return self._lib.Capacitors_Get_AvailableSteps()

    def IsDelta(self, *args):
        """
        Delta connection or wye?

        Original COM help: https://opendss.epri.com/IsDelta.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Capacitors_Get_IsDelta()

        # Setter
        (Value,) = args
        self._lib.Capacitors_Set_IsDelta(Value)

    def NumSteps(self, *args):
        """
        Number of steps (default 1) for distributing and switching the total bank kVAR.

        Original COM help: https://opendss.epri.com/NumSteps.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Capacitors_Get_NumSteps()

        # Setter
        (Value,) = args
        self._lib.Capacitors_Set_NumSteps(Value)

    def States(self, *args):
        """
        An array of integers [0..NumSteps-1] indicating state of each step. If the read value is -1 an error has occurred.

        Original COM help: https://opendss.epri.com/States.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Capacitors_Get_States_GR()

        # Setter
        (Value,) = args
        Value, ValuePtr, ValueCount = self._prepare_int32_array(Value)
        self._lib.Capacitors_Set_States(ValuePtr, ValueCount)

    def kV(self, *args):
        """
        Bank kV rating. Use LL for 2 or 3 phases, or actual can rating for 1 phase.

        Original COM help: https://opendss.epri.com/kV.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Capacitors_Get_kV()

        # Setter
        (Value,) = args
        self._lib.Capacitors_Set_kV(Value)

    def kvar(self, *args):
        """Total bank KVAR, distributed equally among phases and steps."""
        # Getter
        if len(args) == 0:
            return self._lib.Capacitors_Get_kvar()

        # Setter
        (Value,) = args
        self._lib.Capacitors_Set_kvar(Value)


_Capacitors = ICapacitors(api_util)

# For backwards compatibility, bind to the default instance
AddStep = _Capacitors.AddStep
AllNames = _Capacitors.AllNames
AvailableSteps = _Capacitors.AvailableSteps
Close = _Capacitors.Close
Count = _Capacitors.Count
First = _Capacitors.First
Idx = _Capacitors.Idx
IsDelta = _Capacitors.IsDelta
Name = _Capacitors.Name
Next = _Capacitors.Next
NumSteps = _Capacitors.NumSteps
Open = _Capacitors.Open
States = _Capacitors.States
SubtractStep = _Capacitors.SubtractStep
kV = _Capacitors.kV
kvar = _Capacitors.kvar
_columns = _Capacitors._columns
__all__ = [
    "AddStep",
    "AllNames",
    "AvailableSteps",
    "Close",
    "Count",
    "First",
    "Idx",
    "IsDelta",
    "Name",
    "Next",
    "NumSteps",
    "Open",
    "States",
    "SubtractStep",
    "kV",
    "kvar",
]



from ._utils import api_util
from .Bases import Iterable

class IGICSources(Iterable):
    __slots__ = []

    __name__ = "GICSources"
    _api_prefix = "GICSources"
    _columns = [
        "Name",
        "Idx",
        "Phases",
        "Bus1",
        "Bus2",
        "EN",
        "EE",
        "Lat1",
        "Lat2",
        "Lon1",
        "Lon2",
        "Volts",
    ]

    def Bus1(self):
        """First bus name of GICSource (Created name)"""
        return self._lib.GICSources_Get_Bus1()

    def Bus2(self):
        """Second bus name"""
        return self._lib.GICSources_Get_Bus2()

    def Phases(self, *args):
        """Number of Phases, this GICSource element."""
        # Getter
        if len(args) == 0:
            return self._lib.GICSources_Get_Phases()

        # Setter
        (Value,) = args
        self._lib.GICSources_Set_Phases(Value)

    def EN(self, *args):
        """Northward E Field V/km"""
        # Getter
        if len(args) == 0:
            return self._lib.GICSources_Get_EN()

        # Setter
        (Value,) = args
        self._lib.GICSources_Set_EN(Value)

    def EE(self, *args):
        """Eastward E Field, V/km"""
        # Getter
        if len(args) == 0:
            return self._lib.GICSources_Get_EE()

        # Setter
        (Value,) = args
        self._lib.GICSources_Set_EE(Value)

    def Lat1(self, *args):
        """Latitude of Bus1 (degrees)"""
        # Getter
        if len(args) == 0:
            return self._lib.GICSources_Get_Lat1()

        # Setter
        (Value,) = args
        self._lib.GICSources_Set_Lat1(Value)

    def Lat2(self, *args):
        """Latitude of Bus2 (degrees)"""
        # Getter
        if len(args) == 0:
            return self._lib.GICSources_Get_Lat2()

        # Setter
        (Value,) = args
        self._lib.GICSources_Set_Lat2(Value)

    def Lon1(self, *args):
        """Longitude of Bus1 (Degrees)"""
        # Getter
        if len(args) == 0:
            return self._lib.GICSources_Get_Lon1()

        # Setter
        (Value,) = args
        self._lib.GICSources_Set_Lon1(Value)

    def Lon2(self, *args):
        """Longitude of Bus2 (Degrees)"""
        # Getter
        if len(args) == 0:
            return self._lib.GICSources_Get_Lon2()

        # Setter
        (Value,) = args
        self._lib.GICSources_Set_Lon2(Value)

    def Volts(self, *args):
        """Specify dc voltage directly"""
        # Getter
        if len(args) == 0:
            return self._lib.GICSources_Get_Volts()

        # Setter
        (Value,) = args
        self._lib.GICSources_Set_Volts(Value)


_GICSources = IGICSources(api_util)

# For backwards compatibility, bind to the default instance
AllNames = _GICSources.AllNames
Bus1 = _GICSources.Bus1
Bus2 = _GICSources.Bus2
Count = _GICSources.Count
EE = _GICSources.EE
EN = _GICSources.EN
First = _GICSources.First
Idx = _GICSources.Idx
Lat1 = _GICSources.Lat1
Lat2 = _GICSources.Lat2
Lon1 = _GICSources.Lon1
Lon2 = _GICSources.Lon2
Name = _GICSources.Name
Next = _GICSources.Next
Phases = _GICSources.Phases
Volts = _GICSources.Volts
_columns = _GICSources._columns
__all__ = [
    "AllNames",
    "Bus1",
    "Bus2",
    "Count",
    "EE",
    "EN",
    "First",
    "Idx",
    "Lat1",
    "Lat2",
    "Lon1",
    "Lon2",
    "Name",
    "Next",
    "Phases",
    "Volts",
]



from ._utils import api_util
from .Bases import Iterable
from dss import LoadStatus, LoadModels


class ILoads(Iterable):
    __slots__ = []

    __name__ = "Loads"
    _api_prefix = "Loads"
    _columns = [
        "Name",
        "Idx",
        "Phases",
        "Class",
        "Model",
        "NumCust",
        "IsDelta",
        "Rneut",
        "Xneut",
        "PF",
        "ZipV",
        "CVRCurve",
        "CVRvars",
        "CVRwatts",
        "CFactor",
        "Growth",
        "Daily",
        "Duty",
        "Yearly",
        "PctMean",
        "PctStdDev",
        "RelWeighting",
        "Spectrum",
        "Status",
        "VminNorm",
        "VminEmerg",
        "Vminpu",
        "Vmaxpu",
        "kV",
        "kW",
        "kVABase",
        "kvar",
        "kWh",
        "kWhDays",
        "AllocationFactor",
        "XfkVA",
        "puSeriesRL",
        "Sensor",
    ]

    def AllocationFactor(self, *args):
        """
        Factor for allocating loads by connected xfkva

        Original COM help: https://opendss.epri.com/AllocationFactor.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Loads_Get_AllocationFactor()

        # Setter
        (Value,) = args
        self._lib.Loads_Set_AllocationFactor(Value)

    def CVRCurve(self, *args):
        """
        Name of a loadshape with both Mult and Qmult, for CVR factors as a function of time.

        Original COM help: https://opendss.epri.com/CVRcurve.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Loads_Get_CVRcurve()

        # Setter
        (Value,) = args
        self._lib.Loads_Set_CVRcurve(Value)

    def CVRvars(self, *args):
        """
        Percent reduction in Q for percent reduction in V. Must be used with dssLoadModelCVR.

        Original COM help: https://opendss.epri.com/CVRvars.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Loads_Get_CVRvars()

        # Setter
        (Value,) = args
        self._lib.Loads_Set_CVRvars(Value)

    def CVRwatts(self, *args):
        """
        Percent reduction in P for percent reduction in V. Must be used with dssLoadModelCVR.

        Original COM help: https://opendss.epri.com/CVRwatts.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Loads_Get_CVRwatts()

        # Setter
        (Value,) = args
        self._lib.Loads_Set_CVRwatts(Value)

    def CFactor(self, *args):
        """
        CFactor relates average to peak kw.  Used for allocation with kwh and kwhdays

        Original COM help: https://opendss.epri.com/Cfactor.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Loads_Get_Cfactor()

        # Setter
        (Value,) = args
        self._lib.Loads_Set_Cfactor(Value)

    def Class(self, *args):
        """
        Code number used to separate loads by class or group. No effect on the solution.

        Original COM help: https://opendss.epri.com/Class.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Loads_Get_Class_()

        # Setter
        (Value,) = args
        self._lib.Loads_Set_Class_(Value)

    def Growth(self, *args):
        """
        Name of the growthshape curve for yearly load growth factors.

        Original COM help: https://opendss.epri.com/Growth.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Loads_Get_Growth()

        # Setter
        (Value,) = args
        self._lib.Loads_Set_Growth(Value)

    def IsDelta(self, *args):
        """
        Delta loads are connected line-to-line.

        Original COM help: https://opendss.epri.com/IsDelta1.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Loads_Get_IsDelta()

        # Setter
        (Value,) = args
        self._lib.Loads_Set_IsDelta(Value)

    def Model(self, *args):
        """
        The Load Model defines variation of P and Q with voltage.

        Original COM help: https://opendss.epri.com/Model1.html
        """
        # Getter
        if len(args) == 0:
            return LoadModels(self._lib.Loads_Get_Model())

        # Setter
        (Value,) = args
        self._lib.Loads_Set_Model(Value)

    def NumCust(self, *args):
        """
        Number of customers in this load, defaults to one.

        Original COM help: https://opendss.epri.com/NumCust1.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Loads_Get_NumCust()

        # Setter
        (Value,) = args
        self._lib.Loads_Set_NumCust(Value)

    def PF(self, *args):
        """
        Get or set Power Factor for Active Load. Specify leading PF as negative. Updates kvar based on present value of kW

        Original COM help: https://opendss.epri.com/PF1.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Loads_Get_PF()

        # Setter
        (Value,) = args
        self._lib.Loads_Set_PF(Value)

    def PctMean(self, *args):
        """
        Average percent of nominal load in Monte Carlo studies; only if no loadshape defined for this load.

        Original COM help: https://opendss.epri.com/PctMean.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Loads_Get_PctMean()

        # Setter
        (Value,) = args
        self._lib.Loads_Set_PctMean(Value)

    def PctStdDev(self, *args):
        """
        Percent standard deviation for Monte Carlo load studies; if there is no loadshape assigned to this load.

        Original COM help: https://opendss.epri.com/PctStdDev.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Loads_Get_PctStdDev()

        # Setter
        (Value,) = args
        self._lib.Loads_Set_PctStdDev(Value)

    def RelWeighting(self, *args):
        """
        Relative Weighting factor for the active load.

        This value is used in reliability methods.

        Original COM help: https://opendss.epri.com/RelWeight.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Loads_Get_RelWeight()

        # Setter
        (Value,) = args
        self._lib.Loads_Set_RelWeight(Value)

    def Rneut(self, *args):
        """
        Neutral resistance for wye-connected loads.

        Original COM help: https://opendss.epri.com/Rneut.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Loads_Get_Rneut()

        # Setter
        (Value,) = args
        self._lib.Loads_Set_Rneut(Value)

    def Spectrum(self, *args):
        """
        Name of harmonic current spectrum shape.

        Original COM help: https://opendss.epri.com/Spectrum.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Loads_Get_Spectrum()

        # Setter
        (Value,) = args
        self._lib.Loads_Set_Spectrum(Value)

    def Status(self, *args):
        """
        Response to load multipliers: Fixed (growth only), Exempt (no LD curve), Variable (all).

        Original COM help: https://opendss.epri.com/Status.html
        """
        # Getter
        if len(args) == 0:
            return LoadStatus(self._lib.Loads_Get_Status())

        # Setter
        (Value,) = args
        self._lib.Loads_Set_Status(Value)

    def Vmaxpu(self, *args):
        """
        Maximum per-unit voltage to use the load model. Above this, constant Z applies.

        Original COM help: https://opendss.epri.com/Vmaxpu1.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Loads_Get_Vmaxpu()

        # Setter
        (Value,) = args
        self._lib.Loads_Set_Vmaxpu(Value)

    def VminEmerg(self, *args):
        """
        Minimum voltage for unserved energy (UE) evaluation.

        Original COM help: https://opendss.epri.com/Vminemerg.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Loads_Get_Vminemerg()

        # Setter
        (Value,) = args
        self._lib.Loads_Set_Vminemerg(Value)

    def VminNorm(self, *args):
        """
        Minimum voltage for energy exceeding normal (EEN) evaluations.

        Original COM help: https://opendss.epri.com/Vminnorm.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Loads_Get_Vminnorm()

        # Setter
        (Value,) = args
        self._lib.Loads_Set_Vminnorm(Value)

    def Vminpu(self, *args):
        """
        Minimum voltage to apply the load model. Below this, constant Z is used.

        Original COM help: https://opendss.epri.com/Vminpu1.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Loads_Get_Vminpu()

        # Setter
        (Value,) = args
        self._lib.Loads_Set_Vminpu(Value)

    def Xneut(self, *args):
        """
        Neutral reactance for wye-connected loads.

        Original COM help: https://opendss.epri.com/Xneut.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Loads_Get_Xneut()

        # Setter
        (Value,) = args
        self._lib.Loads_Set_Xneut(Value)

    def Yearly(self, *args):
        """
        Name of yearly duration loadshape

        Original COM help: https://opendss.epri.com/Yearly.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Loads_Get_Yearly()

        # Setter
        (Value,) = args
        self._lib.Loads_Set_Yearly(Value)

    def ZipV(self, *args):
        """
        Array of 7 doubles with values for ZIPV property of the load object

        Original COM help: https://opendss.epri.com/ZIPV.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Loads_Get_ZIPV_GR()

        # Setter
        (Value,) = args
        Value, ValuePtr, ValueCount = self._prepare_float64_array(Value)
        self._lib.Loads_Set_ZIPV(ValuePtr, ValueCount)

    def Daily(self, *args):
        """
        Name of the loadshape for a daily load profile.

        Original COM help: https://opendss.epri.com/daily.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Loads_Get_daily()

        # Setter
        (Value,) = args
        self._lib.Loads_Set_daily(Value)

    def Duty(self, *args):
        """
        Name of the loadshape for a duty cycle simulation.

        Original COM help: https://opendss.epri.com/duty.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Loads_Get_duty()

        # Setter
        (Value,) = args
        self._lib.Loads_Set_duty(Value)

    def kV(self, *args):
        """
        kV rating for active Load. For 2 or more phases set Line-Line kV. Else actual kV across terminals.

        Original COM help: https://opendss.epri.com/kV2.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Loads_Get_kV()

        # Setter
        (Value,) = args
        self._lib.Loads_Set_kV(Value)

    def kW(self, *args):
        """
        Set kW for active Load. Updates kvar based on present PF.

        Original COM help: https://opendss.epri.com/kW1.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Loads_Get_kW()

        # Setter
        (Value,) = args
        self._lib.Loads_Set_kW(Value)

    def kVABase(self, *args):
        """
        Base load kva. Also defined kw and kvar or pf input, or load allocation by kwh or xfkva.

        Original COM help: https://opendss.epri.com/kva.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Loads_Get_kva()

        # Setter
        (Value,) = args
        self._lib.Loads_Set_kva(Value)

    def kvar(self, *args):
        """
        Reactive power in kvar for active Load. If set, updates PF based on present kW.

        Original COM help: https://opendss.epri.com/kvar1.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Loads_Get_kvar()

        # Setter
        (Value,) = args
        self._lib.Loads_Set_kvar(Value)

    def kWh(self, *args):
        """
        kWh billed for this period. Can be used with Cfactor for load allocation.

        Original COM help: https://opendss.epri.com/kwh.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Loads_Get_kwh()

        # Setter
        (Value,) = args
        self._lib.Loads_Set_kwh(Value)

    def kWhDays(self, *args):
        """
        Length of kWh billing period for average demand calculation. Default 30.

        Original COM help: https://opendss.epri.com/kwhdays.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Loads_Get_kwhdays()

        # Setter
        (Value,) = args
        self._lib.Loads_Set_kwhdays(Value)

    def puSeriesRL(self, *args):
        """
        Percent of Load that is modeled as series R-L for harmonics studies

        Original COM help: https://opendss.epri.com/pctSeriesRL.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Loads_Get_pctSeriesRL()

        # Setter
        (Value,) = args
        self._lib.Loads_Set_pctSeriesRL(Value)

    def XfkVA(self, *args):
        """
        Rated service transformer kVA for load allocation, using AllocationFactor. Affects kW, kvar, and pf.

        Original COM help: https://opendss.epri.com/xfkVA.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Loads_Get_xfkVA()

        # Setter
        (Value,) = args
        self._lib.Loads_Set_xfkVA(Value)

    def Sensor(self):
        """
        Name of the sensor monitoring this load.

        Original COM help: https://opendss.epri.com/Sensor.html
        """
        return self._lib.Loads_Get_Sensor()

    def Phases(self, *args):
        """
        Number of phases

        **(API Extension)**
        """
        # Getter
        if len(args) == 0:
            return self._lib.Loads_Get_Phases()

        # Setter
        (Value,) = args
        self._lib.Loads_Set_Phases(Value)


_Loads = ILoads(api_util)

# For backwards compatibility, bind to the default instance
AllNames = _Loads.AllNames
AllocationFactor = _Loads.AllocationFactor
CFactor = _Loads.CFactor
CVRCurve = _Loads.CVRCurve
CVRvars = _Loads.CVRvars
CVRwatts = _Loads.CVRwatts
Class = _Loads.Class
Count = _Loads.Count
Daily = _Loads.Daily
Duty = _Loads.Duty
First = _Loads.First
Growth = _Loads.Growth
Idx = _Loads.Idx
IsDelta = _Loads.IsDelta
Model = _Loads.Model
Name = _Loads.Name
Next = _Loads.Next
NumCust = _Loads.NumCust
PF = _Loads.PF
PctMean = _Loads.PctMean
PctStdDev = _Loads.PctStdDev
Phases = _Loads.Phases
RelWeighting = _Loads.RelWeighting
Rneut = _Loads.Rneut
Sensor = _Loads.Sensor
Spectrum = _Loads.Spectrum
Status = _Loads.Status
Vmaxpu = _Loads.Vmaxpu
VminEmerg = _Loads.VminEmerg
VminNorm = _Loads.VminNorm
Vminpu = _Loads.Vminpu
XfkVA = _Loads.XfkVA
Xneut = _Loads.Xneut
Yearly = _Loads.Yearly
ZipV = _Loads.ZipV
kV = _Loads.kV
kVABase = _Loads.kVABase
kW = _Loads.kW
kWh = _Loads.kWh
kWhDays = _Loads.kWhDays
kvar = _Loads.kvar
puSeriesRL = _Loads.puSeriesRL
_columns = _Loads._columns
__all__ = [
    "AllNames",
    "AllocationFactor",
    "CFactor",
    "CVRCurve",
    "CVRvars",
    "CVRwatts",
    "Class",
    "Count",
    "Daily",
    "Duty",
    "First",
    "Growth",
    "Idx",
    "IsDelta",
    "Model",
    "Name",
    "Next",
    "NumCust",
    "PF",
    "PctMean",
    "PctStdDev",
    "Phases",
    "RelWeighting",
    "Rneut",
    "Sensor",
    "Spectrum",
    "Status",
    "Vmaxpu",
    "VminEmerg",
    "VminNorm",
    "Vminpu",
    "XfkVA",
    "Xneut",
    "Yearly",
    "ZipV",
    "kV",
    "kVABase",
    "kW",
    "kWh",
    "kWhDays",
    "kvar",
    "puSeriesRL",
]



import numpy as np
from ._utils import DSSException, api_util
from .Bases import Iterable


class IMonitors(Iterable):
    __slots__ = []

    __name__ = "Monitors"
    _api_prefix = "Monitors"
    _columns = [
        "Name",
        "FileVersion",
        "NumChannels",
        "RecordSize",
        "dblFreq",
        "Mode",
        "FileName",
        "Element",
        "Header",
        "Terminal",
        "dblHour",
        "SampleCount",
    ]

    def Channel(self, Index):
        """
        (read-only) Array of float32 for the specified channel (usage: MyArray = DSSMonitor.Channel(i)).
        A Save or SaveAll should be executed first. Done automatically by most standard solution modes.
        Channels start at index 1.

        Original COM help: https://opendss.epri.com/Channel.html
        """
        num_channels = self._lib.Monitors_Get_NumChannels()
        if Index < 1 or Index > num_channels:
            raise DSSException(
                0,
                'Monitors.Channel: Invalid channel index ({}), monitor "{}" has {} channels.'.format(
                    Index, self.Name, num_channels
                ),
            )
        ffi = self._api_util.ffi
        api_util = self._api_util
        api_util.lib_unpatched.Monitors_Get_ByteStream_GR(api_util.ctx)
        api_util._check_for_error()
        ptr, cnt = api_util.gr_int8_pointers
        cnt = cnt[0]
        if cnt == 272:
            return np.zeros((1,), dtype=np.float32)
        ptr = ptr[0]
        record_size = ffi.cast("int32_t*", ptr)[2] + 2
        data = np.frombuffer(ffi.buffer(ptr, cnt), dtype=np.float32, offset=272)
        return data[Index + 1 :: record_size].copy()

    def AsMatrix(self):
        """
        Matrix of the active monitor, containing the hour vector, seconds vector, and all channels (index 2 = channel 1).
        If you need multiple channels, prefer using this function as it processes the monitor byte-stream once.

        **(API Extension)**
        """
        ffi = self._api_util.ffi
        api_util = self._api_util
        api_util.lib_unpatched.Monitors_Get_ByteStream_GR(api_util.ctx)
        api_util._check_for_error()
        ptr, cnt = api_util.gr_int8_pointers
        cnt = cnt[0]
        if cnt == 272:
            return None
        ptr = ptr[0]
        record_size = ffi.cast("int32_t*", ptr)[2] + 2
        data = np.frombuffer(ffi.buffer(ptr, cnt), dtype=np.float32, offset=272)
        data = data.reshape((len(data) // record_size, record_size)).copy()
        return data

    def Process(self):
        """
        Post-process monitor samples taken so far, e.g., Pst for mode=4.

        Original COM help: https://opendss.epri.com/Process.html
        """
        self._lib.Monitors_Process()

    def ProcessAll(self):
        """
        Post-process all monitor samples taken so far, e.g., Pst for mode=4.

        Original COM help: https://opendss.epri.com/ProcessAll.html
        """
        self._lib.Monitors_ProcessAll()

    def Reset(self):
        """
        Reset active Monitor object.

        Original COM help: https://opendss.epri.com/Reset3.html
        """
        self._lib.Monitors_Reset()

    def ResetAll(self):
        """
        Reset all Monitor objects.

        Original COM help: https://opendss.epri.com/ResetAll1.html
        """
        self._lib.Monitors_ResetAll()

    def Sample(self):
        """
        Instruct the active Monitor to take a sample of the present state.

        Original COM help: https://opendss.epri.com/Sample2.html
        """
        self._lib.Monitors_Sample()

    def SampleAll(self):
        """
        Instruct all Monitor objects to take a sample of the present state.

        Original COM help: https://opendss.epri.com/SampleAll1.html
        """
        self._lib.Monitors_SampleAll()

    def Save(self):
        """
        Instructs the active monitor to save its current sample buffer to its monitor stream.

        After the data is on the stream, you can access the ByteStream or channel data.

        **Most standard solution modes do this automatically.**

        Original COM help: https://opendss.epri.com/Save1.html
        """
        self._lib.Monitors_Save()

    def SaveAll(self):
        """
        Instructs the all monitor objects to save their current sample buffers to the respective monitor streams.

        **Most standard solution modes do this automatically.**

        Original COM help: https://opendss.epri.com/SaveAll1.html
        """
        self._lib.Monitors_SaveAll()

    def Show(self):
        """
        Convert the monitor data to text and displays it with the text editor.

        Original COM help: https://opendss.epri.com/Show3.html
        """
        self._lib.Monitors_Show()

    def ByteStream(self):
        """
        Byte Array containing monitor stream values. Make sure a "save" is done first (standard solution modes do this automatically)

        Original COM help: https://opendss.epri.com/ByteStream.html
        """
        return self._lib.Monitors_Get_ByteStream_GR()

    def Element(self, *args):
        """
        Full object name of element being monitored.

        Original COM help: https://opendss.epri.com/Element.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Monitors_Get_Element()

        # Setter
        (Value,) = args
        self._lib.Monitors_Set_Element(Value)

    def FileName(self):
        """
        Name of CSV file associated with active Monitor.

        Original COM help: https://opendss.epri.com/FileName.html
        """
        return self._lib.Monitors_Get_FileName()

    def FileVersion(self):
        """
        Monitor File Version (integer)

        Original COM help: https://opendss.epri.com/FileVersion.html
        """
        return self._lib.Monitors_Get_FileVersion()

    def Header(self):
        """
        Header string;  Array of strings containing Channel names

        Original COM help: https://opendss.epri.com/Header.html
        """
        return self._lib.Monitors_Get_Header()

    def Mode(self, *args):
        """
        Monitor mode (bitmask integer - see DSS Help)

        Original COM help: https://opendss.epri.com/Mode1.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Monitors_Get_Mode()

        # Setter
        (Value,) = args
        self._lib.Monitors_Set_Mode(Value)

    def NumChannels(self):
        """
        Number of Channels in the active Monitor

        Original COM help: https://opendss.epri.com/NumChannels.html
        """
        return self._lib.Monitors_Get_NumChannels()

    def RecordSize(self):
        """
        Size of each record in ByteStream (Integer). Same as NumChannels.

        Original COM help: https://opendss.epri.com/RecordSize.html
        """
        return self._lib.Monitors_Get_RecordSize()

    def SampleCount(self):
        """
        Number of Samples in Monitor at Present

        Original COM help: https://opendss.epri.com/SampleCount.html
        """
        return self._lib.Monitors_Get_SampleCount()

    def Terminal(self, *args):
        """
        Terminal number of element being monitored.

        Original COM help: https://opendss.epri.com/Terminal.html
        """
        # Getter
        if len(args) == 0:
            return self._lib.Monitors_Get_Terminal()

        # Setter
        (Value,) = args
        self._lib.Monitors_Set_Terminal(Value)

    def dblFreq(self):
        """
        Array of doubles containing frequency values for harmonics mode solutions; Empty for time mode solutions (use dblHour)

        Original COM help: https://opendss.epri.com/dblFreq.html
        """
        return self._lib.Monitors_Get_dblFreq_GR()

    def dblHour(self):
        """
        Array of doubles containing time value in hours for time-sampled monitor values; Empty if frequency-sampled values for harmonics solution (see dblFreq)

        Original COM help: https://opendss.epri.com/dblHour.html
        """
        return self._lib.Monitors_Get_dblHour_GR()


_Monitors = IMonitors(api_util)

# For backwards compatibility, bind to the default instance
AllNames = _Monitors.AllNames
AsMatrix = _Monitors.AsMatrix
ByteStream = _Monitors.ByteStream
Channel = _Monitors.Channel
Count = _Monitors.Count
Element = _Monitors.Element
FileName = _Monitors.FileName
FileVersion = _Monitors.FileVersion
First = _Monitors.First
Header = _Monitors.Header
Idx = _Monitors.Idx
Mode = _Monitors.Mode
Name = _Monitors.Name
Next = _Monitors.Next
NumChannels = _Monitors.NumChannels
Process = _Monitors.Process
ProcessAll = _Monitors.ProcessAll
RecordSize = _Monitors.RecordSize
Reset = _Monitors.Reset
ResetAll = _Monitors.ResetAll
Sample = _Monitors.Sample
SampleAll = _Monitors.SampleAll
SampleCount = _Monitors.SampleCount
Save = _Monitors.Save
SaveAll = _Monitors.SaveAll
Show = _Monitors.Show
Terminal = _Monitors.Terminal
dblFreq = _Monitors.dblFreq
dblHour = _Monitors.dblHour
_columns = _Monitors._columns
__all__ = [
    "AllNames",
    "AsMatrix",
    "ByteStream",
    "Channel",
    "Count",
    "Element",
    "FileName",
    "FileVersion",
    "First",
    "Header",
    "Idx",
    "Mode",
    "Name",
    "Next",
    "NumChannels",
    "Process",
    "ProcessAll",
    "RecordSize",
    "Reset",
    "ResetAll",
    "Sample",
    "SampleAll",
    "SampleCount",
    "Save",
    "SaveAll",
    "Show",
    "Terminal",
    "dblFreq",
    "dblHour",
]



from ._utils import api_util
from .Bases import Base


class ICtrlQueue(Base):
    __slots__ = []

    __name__ = "CtrlQueue"
    _api_prefix = "CtrlQueue"
    _columns = ["Queue", "DeviceHandle", "QueueSize", "ActionCode", "NumActions"]

    def ClearActions(self):
        """
        Clear all actions from the Control Proxy's Action List (they are popped off the list).

        Original COM help: https://opendss.epri.com/ClearActions.html
        """
        self._lib.CtrlQueue_ClearActions()

    def ClearQueue(self):
        """
        Clear the control queue.

        Original COM help: https://opendss.epri.com/ClearQueue.html
        """
        self._lib.CtrlQueue_ClearQueue()

    def Delete(self, ActionHandle):
        """
        Delete an Action from the DSS Control Queue by the handle that is returned when the action is added.

        (The Push function returns the handle.)

        Original COM help: https://opendss.epri.com/Delete.html
        """
        self._lib.CtrlQueue_Delete(ActionHandle)

    def DoAllQueue(self):
        """
        Execute all actions currently on the Control Queue.

        Side effect: clears the queue.

        Original COM help: https://opendss.epri.com/DoAllQueue.html
        """
        self._lib.CtrlQueue_DoAllQueue()

    def Show(self):
        """
        Export the queue to a CSV table and show it.

        Original COM help: https://opendss.epri.com/Show.html
        """
        self._lib.CtrlQueue_Show()

    def ActionCode(self):
        """
        Code for the active action. Integer code to tell the control device what to do.

        Use this to determine what the user-defined controls are supposed to do.
        It can be any 32-bit integer of the user's choosing and is the same value that the control pushed onto the control queue earlier.

        Original COM help: https://opendss.epri.com/ActionCode.html
        """
        return self._lib.CtrlQueue_Get_ActionCode()

    def DeviceHandle(self):
        """
        Handle (User defined) to device that must act on the pending action.

        The user-written code driving the interface may support more than one
        control element as necessary to perform the simulation. This handle is
        an index returned to the user program that lets the program know which
        control is to perform the active action.

        Original COM help: https://opendss.epri.com/DeviceHandle.html
        """
        return self._lib.CtrlQueue_Get_DeviceHandle()

    def NumActions(self):
        """
        Number of Actions on the current action list (that have been popped off the control queue by CheckControlActions)

        Original COM help: https://opendss.epri.com/NumActions.html
        """
        return self._lib.CtrlQueue_Get_NumActions()

    def Push(self, Hour, Seconds, ActionCode, DeviceHandle):
        """
        Push a control action onto the DSS control queue by time, action code, and device handle (user defined). Returns Control Queue handle.

        Original COM help: https://opendss.epri.com/Push.html
        """
        return self._lib.CtrlQueue_Push(Hour, Seconds, ActionCode, DeviceHandle)

    def PopAction(self):
        """
        Pops next action off the action list and makes it the active action. Returns zero if none.

        Original COM help: https://opendss.epri.com/PopAction.html
        """
        return self._lib.CtrlQueue_Get_PopAction()

    def Queue(self):
        """
        Array of strings containing the entire queue in CSV format

        Original COM help: https://opendss.epri.com/Queue.html
        """
        return self._lib.CtrlQueue_Get_Queue()

    def QueueSize(self):
        """
        Number of items on the OpenDSS control Queue

        Original COM help: https://opendss.epri.com/QueueSize.html
        """
        return self._lib.CtrlQueue_Get_QueueSize()

    def Action(self, Param1):
        """
        (write-only) Set the active action by index

        Original COM help: https://opendss.epri.com/Action.html
        """
        self._lib.CtrlQueue_Set_Action(Param1)


_CtrlQueue = ICtrlQueue(api_util)

# For backwards compatibility, bind to the default instance
Action = _CtrlQueue.Action
ActionCode = _CtrlQueue.ActionCode
ClearActions = _CtrlQueue.ClearActions
ClearQueue = _CtrlQueue.ClearQueue
Delete = _CtrlQueue.Delete
DeviceHandle = _CtrlQueue.DeviceHandle
DoAllQueue = _CtrlQueue.DoAllQueue
NumActions = _CtrlQueue.NumActions
PopAction = _CtrlQueue.PopAction
Push = _CtrlQueue.Push
Queue = _CtrlQueue.Queue
QueueSize = _CtrlQueue.QueueSize
Show = _CtrlQueue.Show
_columns = _CtrlQueue._columns
__all__ = [
    "Action",
    "ActionCode",
    "ClearActions",
    "ClearQueue",
    "Delete",
    "DeviceHandle",
    "DoAllQueue",
    "NumActions",
    "PopAction",
    "Push",
    "Queue",
    "QueueSize",
    "Show",
]
